@@ -1,13 +1,12 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2020-2022 NXP
+# Copyright 2020-2023 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
 """Console script for pfr."""
 
-import io
 import logging
 import os
 import sys
@@ -15,21 +14,22 @@ from typing import Optional, Tuple, Type, Union
 
 import click
 from click_option_group import MutuallyExclusiveOptionGroup, optgroup
-from ruamel.yaml import YAML
 
-from spsdk import SPSDK_YML_INDENT, pfr
+from spsdk import pfr
 from spsdk.apps.elftosb_utils.sb_31_helper import RootOfTrustInfo
 from spsdk.apps.utils.common_cli_options import (
     FC,
-    CommandsTreeGroup,
+    CommandsTreeGroupAliasedGetCfgTemplate,
     isp_interfaces,
     spsdk_apps_common_options,
 )
 from spsdk.apps.utils.utils import MBootInterface, catch_spsdk_error, format_raw_data, get_interface
 from spsdk.crypto.loaders import extract_public_keys
 from spsdk.mboot import McuBoot
-from spsdk.pfr.exceptions import SPSDKError, SPSDKPfrConfigError
+from spsdk.pfr.exceptions import SPSDKError, SPSDKPfrConfigError, SPSDKPfrError
+from spsdk.pfr.pfrc import Pfrc
 from spsdk.utils.misc import find_file, load_configuration, size_fmt, write_file
+from spsdk.utils.schema_validator import ConfigTemplate
 
 PFRArea = Union[Type[pfr.CMPA], Type[pfr.CFPA]]
 logger = logging.getLogger(__name__)
@@ -40,8 +40,7 @@ def _store_output(data: Union[str, bytes], path: Optional[str], mode: str = "w")
     if path is None:
         click.echo(data)
     else:
-        with open(path, mode) as f:
-            f.write(data)
+        write_file(data, path=path, mode=mode)
 
 
 def _get_pfr_class(area_name: str) -> PFRArea:
@@ -74,7 +73,7 @@ def pfr_device_type_options(options: FC) -> FC:
     return options
 
 
-@click.group(name="pfr", no_args_is_help=True, cls=CommandsTreeGroup)
+@click.group(name="pfr", no_args_is_help=True, cls=CommandsTreeGroupAliasedGetCfgTemplate)
 @spsdk_apps_common_options
 def main(log_level: int) -> int:
     """Utility for generating and parsing Protected Flash Region data (CMPA, CFPA)."""
@@ -82,7 +81,7 @@ def main(log_level: int) -> int:
     return 0
 
 
-@main.command(name="get-cfg-template", no_args_is_help=True)
+@main.command(name="get-template", no_args_is_help=True)
 @pfr_device_type_options
 @click.option(
     "-o",
@@ -92,15 +91,11 @@ def main(log_level: int) -> int:
     help="Save the output into a file instead of console",
 )
 @click.option("-f", "--full", is_flag=True, help="Show full config, including computed values")
-def get_cfg_template(device: str, revision: str, area: str, output: str, full: bool) -> None:
+def get_template(device: str, revision: str, area: str, output: str, full: bool) -> None:
     """Generate user configuration template file."""
     pfr_obj = _get_pfr_class(area)(device=device, revision=revision)
-    yaml = YAML(pure=True)
-    yaml.indent(sequence=SPSDK_YML_INDENT * 2, offset=SPSDK_YML_INDENT)
     data = pfr_obj.get_yaml_config(not full)
-    stream = io.StringIO()
-    yaml.dump(data, stream)
-    yaml_data = stream.getvalue()
+    yaml_data = ConfigTemplate.convert_cm_to_yaml(data)
     _store_output(yaml_data, output)
 
 
@@ -153,13 +148,14 @@ def parse_binary(
         show_diff=show_diff,
     )
     _store_output(yaml_data, output)
+    click.echo(f"Success. (PFR: {binary} has been parsed and stored into {output}.)")
 
 
 def _parse_binary_data(
     data: bytes,
     device: str,
     area: str,
-    revision: str = None,
+    revision: Optional[str] = None,
     show_calc: bool = False,
     show_diff: bool = False,
 ) -> str:
@@ -176,11 +172,7 @@ def _parse_binary_data(
     pfr_obj = _get_pfr_class(area)(device=device, revision=revision)
     pfr_obj.parse(data)
     parsed = pfr_obj.get_yaml_config(exclude_computed=not show_calc, diff=show_diff)
-    yaml = YAML()
-    yaml.indent(sequence=4, offset=2)
-    stream = io.StringIO()
-    yaml.dump(parsed, stream)
-    yaml_data = stream.getvalue()
+    yaml_data = ConfigTemplate.convert_cm_to_yaml(parsed)
     return yaml_data
 
 
@@ -226,6 +218,13 @@ def _parse_binary_data(
     "--password",
     help="Password when using Encrypted private keys as --secret-file",
 )
+@click.option(
+    "-x",
+    "--force",
+    is_flag=True,
+    default=False,
+    help="Force to generate binary even the config validation fails.",
+)
 def generate_binary(
     output: str,
     user_config_file: str,
@@ -234,6 +233,7 @@ def generate_binary(
     elf2sb_config: str,
     secret_file: Tuple[str],
     password: str,
+    force: bool,
 ) -> None:
     """Generate binary data."""
     pfr_config = pfr.PfrConfiguration(user_config_file)
@@ -242,7 +242,28 @@ def generate_binary(
         raise SPSDKPfrConfigError(
             f"The configuration file is not valid. The reason is: {invalid_reason}"
         )
-    assert pfr_config.type
+    area = pfr_config.type
+    assert area
+    pfr_obj = _get_pfr_class(area)(device=pfr_config.device, revision=pfr_config.revision)
+    pfr_obj.set_config(pfr_config, raw=not calc_inverse)
+    if pfr_config.device in Pfrc.get_supported_families():
+        try:
+            pfrc = Pfrc(
+                cmpa=pfr_obj if area.lower() == "cmpa" else None,  # type: ignore
+                cfpa=pfr_obj if area.lower() == "cfpa" else None,  # type: ignore
+            )
+            rules = pfrc.validate_brick_conditions()
+        except (SPSDKPfrConfigError, SPSDKPfrError) as e:
+            logger.debug(f"PFRC unexpectedly failed: {e}")
+        else:
+            log_text = f"PFRC results: passed: {len(rules[0])}, failed: {len(rules[1])}, ignored: {len(rules[2])}"
+            if rules[1]:
+                if force:
+                    logger.warning(log_text)
+                else:
+                    raise SPSDKError(log_text)
+            else:
+                logger.debug(log_text)
     root_of_trust = None
     keys = None
     if elf2sb_config:
@@ -253,16 +274,13 @@ def generate_binary(
         root_of_trust = tuple((find_file(x, search_paths=[elf2sb_config_dir]) for x in public_keys))
     if secret_file:
         root_of_trust = secret_file
-    area = pfr_config.type
     if area.lower() == "cmpa" and root_of_trust:
         keys = extract_public_keys(root_of_trust, password)
-    pfr_obj = _get_pfr_class(area)(device=pfr_config.device, revision=pfr_config.revision)
     if not pfr_config.revision:
         pfr_config.revision = pfr_obj.revision
-    pfr_obj.set_config(pfr_config, raw=not calc_inverse)
-
     data = pfr_obj.export(add_seal=add_seal, keys=keys)
     _store_output(data, output, "wb")
+    click.echo(f"Success. (PFR binary has been generated into {output}.)")
 
 
 @main.command(name="info", no_args_is_help=True)
@@ -302,7 +320,14 @@ def devices() -> None:
 
 
 @main.command(name="write", no_args_is_help=True)
-@isp_interfaces(uart=True, usb=True, lpcusbsio=True, buspal=True, json_option=False)
+@isp_interfaces(
+    uart=True,
+    usb=True,
+    lpcusbsio=True,
+    buspal=True,
+    json_option=False,
+    use_long_timeout_option=True,
+)
 @pfr_device_type_options
 @click.option(
     "-b",
@@ -323,11 +348,6 @@ def write(
     binary: str,
 ) -> None:
     """Write PFR page to the device."""
-    # silence warning about missing revision in 'normal' logging mode
-    if not logger.isEnabledFor(logging.INFO):
-        pfr_logger = logging.getLogger("spsdk.pfr")
-        pfr_logger.setLevel(level=logging.ERROR)
-
     pfr_obj = _get_pfr_class(area)(device=device, revision=revision)
     pfr_page_address = pfr_obj.config.get_address(device)
     pfr_page_length = pfr_obj.BINARY_SIZE
@@ -353,7 +373,14 @@ def write(
 
 
 @main.command(name="read", no_args_is_help=True)
-@isp_interfaces(uart=True, usb=True, lpcusbsio=True, buspal=True, json_option=False)
+@isp_interfaces(
+    uart=True,
+    usb=True,
+    lpcusbsio=True,
+    buspal=True,
+    json_option=False,
+    use_long_timeout_option=True,
+)
 @pfr_device_type_options
 @click.option(
     "-o",
@@ -396,11 +423,6 @@ def read(
     show_calc: bool,
 ) -> None:
     """Read PFR page from the device."""
-    # silence warning about missing revision in 'normal' logging mode
-    if not logger.isEnabledFor(logging.INFO):
-        pfr_logger = logging.getLogger("spsdk.pfr")
-        pfr_logger.setLevel(level=logging.ERROR)
-
     pfr_obj = _get_pfr_class(area)(device=device, revision=revision)
     pfr_page_address = pfr_obj.config.get_address(device)
     pfr_page_length = pfr_obj.BINARY_SIZE
