@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2020-2021 NXP
+# Copyright 2020-2023 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
@@ -11,64 +11,29 @@ import inspect
 import json
 import logging
 import sys
+from typing import Optional
 
 import click
-from click_option_group import MutuallyExclusiveOptionGroup, optgroup
 
-from spsdk import __version__ as spsdk_version
-from spsdk.apps.utils import INT, catch_spsdk_error, format_raw_data, get_interface
+from spsdk.apps.utils.common_cli_options import (
+    CommandsTreeGroup,
+    isp_interfaces,
+    spsdk_apps_common_options,
+)
+from spsdk.apps.utils.utils import (
+    INT,
+    SPSDKAppError,
+    catch_spsdk_error,
+    format_raw_data,
+    get_interface,
+)
 from spsdk.sdp import SDP
 from spsdk.sdp.commands import ResponseValue
 
 
-@click.group(no_args_is_help=True)
-@optgroup.group("Interface configuration", cls=MutuallyExclusiveOptionGroup)
-@optgroup.option(
-    "-p",
-    "--port",
-    metavar="COM[,speed]",
-    help="Serial port.",
-)
-@optgroup.option(
-    "-u",
-    "--usb",
-    metavar="VID,PID",
-    help="""USB device identifier.
-    Following formats are supported: <vid>, <vid:pid> or <vid,pid>, device/instance path, device name.
-    <vid>: hex or dec string; e.g. 0x0AB12, 43794.
-    <vid/pid>: hex or dec string; e.g. 0x0AB12:0x123, 1:3451.
-    Use 'nxpdevscan' utility to list connected device names.
-""",
-)
-@click.option(
-    "-j",
-    "--json",
-    "use_json",
-    is_flag=True,
-    help="Use JSON output",
-)
-@click.option(
-    "-v",
-    "--verbose",
-    "log_level",
-    flag_value=logging.INFO,
-    help="Displays more verbose output",
-)
-@click.option(
-    "-d",
-    "--debug",
-    "log_level",
-    flag_value=logging.DEBUG,
-    help="Display debugging info",
-)
-@click.option(
-    "-t",
-    "--timeout",
-    metavar="<ms>",
-    help="Set packet timeout in milliseconds",
-    default=5000,
-)
-@click.version_option(spsdk_version, "--version")
+@click.group(name="sdphost", no_args_is_help=True, cls=CommandsTreeGroup)
+@isp_interfaces(uart=True, usb=True, is_sdp=True, default_timeout=10000)
+@spsdk_apps_common_options
 @click.pass_context
 def main(
     ctx: click.Context,
@@ -78,17 +43,14 @@ def main(
     log_level: int,
     timeout: int,
 ) -> int:
-    """Utility for communication with ROM on i.MX targets."""
+    """Utility for communication with ROM on i.MX targets using SDP protocol (i.MX RT1xxx)."""
     logging.basicConfig(level=log_level or logging.WARNING)
-    # if --help is provided anywhere on commandline, skip interface lookup and display help message
-    if "--help " in sys.argv:
-        port, usb = None, None  # type: ignore
-    ctx.obj = {
-        "interface": get_interface(module="sdp", port=port, usb=usb, timeout=timeout)
-        if port or usb
-        else None,
-        "use_json": use_json,
-    }
+    # if --help is provided anywhere on command line, skip interface lookup and display help message
+    if "--help" not in sys.argv[1:]:
+        ctx.obj = {
+            "interface": get_interface(module="sdp", port=port, usb=usb, timeout=timeout),
+            "use_json": use_json,
+        }
     return 0
 
 
@@ -148,8 +110,8 @@ def write_file(ctx: click.Context, address: int, bin_file: click.File, count: in
 
 @main.command()
 @click.argument("address", type=INT(), required=True)
-@click.argument("item_length", type=int, required=False, default=32, metavar="[FORMAT]")
-@click.argument("count", type=int, required=False, default=None)
+@click.argument("item_length", type=INT(), required=False, default=32, metavar="[FORMAT]")
+@click.argument("count", type=INT(), required=False, default=None)
 @click.argument("file", type=click.File("wb"), required=False)
 @click.option("-h", "--use-hexdump", is_flag=True, default=False, help="Use hexdump format")
 @click.pass_context
@@ -176,10 +138,9 @@ def read_register(
     with SDP(ctx.obj["interface"]) as sdp:
         response = sdp.read_safe(address, count, item_length)
     if not response:
-        click.echo(
+        raise SPSDKAppError(
             f"Error: invalid sub-command or arguments 'read-register {address:#8X} {item_length} {count}'"
         )
-        sys.exit(1)
     if file:
         file.write(response)  # type: ignore
     else:
@@ -187,8 +148,31 @@ def read_register(
     display_output([], sdp.hab_status, ctx.obj["use_json"])
 
 
+@click.argument("baudrate", type=INT(), required=True)
+@main.command()
+@click.pass_context
+def set_baudrate(ctx: click.Context, baudrate: int) -> None:
+    """Configures UART baudrate.
+
+    The SDP command SET_BAUDRATE is used by the host to configure the UART
+    baudrate on the device side. The default baudrate is 115200.
+    Please note that this command is not supported on all devices.
+
+    \b
+    BAUDRATE - baudrate to be set
+    """
+    with SDP(ctx.obj["interface"]) as sdp:
+        response = sdp.set_baudrate(baudrate)
+    display_output(
+        [response],
+        sdp.hab_status,
+        ctx.obj["use_json"],
+        extra_output=f"Response status = {decode_status_code(response)}.",
+    )
+
+
 def display_output(
-    response: list, status_code: int, use_json: bool = False, extra_output: str = None
+    response: list, status_code: int, use_json: bool = False, extra_output: Optional[str] = None
 ) -> None:
     """Printout the response.
 
@@ -215,13 +199,11 @@ def display_output(
             print(extra_output)
 
 
-def decode_status_code(status_code: int = None) -> str:
+def decode_status_code(status_code: Optional[int] = None) -> str:
     """Returns a stringified representation of status code.
 
     :param status_code: SDP status code
-    :type status_code: int
     :return: stringified representation
-    :rtype: str
     """
     if not status_code:
         return "UNKNOWN ERROR"
@@ -231,8 +213,8 @@ def decode_status_code(status_code: int = None) -> str:
 @catch_spsdk_error
 def safe_main() -> None:
     """Calls the main function."""
-    sys.exit(main())  # pragma: no cover  # pylint: disable=no-value-for-parameter
+    sys.exit(main())  # pylint: disable=no-value-for-parameter
 
 
 if __name__ == "__main__":
-    safe_main()  # pragma: no cover
+    safe_main()
