@@ -1,12 +1,13 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2020-2022 NXP
+# Copyright 2020-2023 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 """Module for key generation and saving keys to file (RSA and ECC)."""
 
 from enum import Enum
+from typing import Optional, Union
 
 from spsdk.crypto import (
     EllipticCurvePrivateKeyWithSerialization,
@@ -14,12 +15,14 @@ from spsdk.crypto import (
     Encoding,
     RSAPrivateKeyWithSerialization,
     RSAPublicKey,
+    RSAPublicNumbers,
     default_backend,
     ec,
     rsa,
     serialization,
 )
 from spsdk.exceptions import SPSDKValueError
+from spsdk.utils.misc import write_file
 
 
 class CurveName(str, Enum):
@@ -92,7 +95,7 @@ def generate_rsa_public_key(
 def save_rsa_private_key(
     private_key: RSAPrivateKeyWithSerialization,
     file_path: str,
-    password: str = None,
+    password: Optional[str] = None,
     encoding: Encoding = Encoding.PEM,
 ) -> None:
     """Save the RSA private key to the given file.
@@ -113,8 +116,7 @@ def save_rsa_private_key(
         else serialization.NoEncryption()
     )
     pem_data = private_key.private_bytes(encoding, serialization.PrivateFormat.PKCS8, enc)
-    with open(file_path, "wb") as f:
-        f.write(pem_data)
+    write_file(pem_data, file_path, mode="wb")
 
 
 def save_rsa_public_key(
@@ -127,8 +129,19 @@ def save_rsa_public_key(
     :param encoding: encoding type, default is PEM
     """
     pem_data = public_key.public_bytes(encoding, serialization.PublicFormat.PKCS1)
-    with open(file_path, "wb") as f:
-        f.write(pem_data)
+    write_file(pem_data, file_path, mode="wb")
+
+
+# pylint: disable=invalid-name  # e,n are commonly used symbols for exponent and modulus
+def recreate_rsa_public_key(e: int, n: int) -> RSAPublicKey:
+    """Recreate RSA public key from Exponent and modulus.
+
+    :param e: Exponent of RSA key.
+    :param n: Modulus of RSA key.
+    :return: RSA public key.
+    """
+    public_numbers = RSAPublicNumbers(e=e, n=n)
+    return public_numbers.public_key()
 
 
 def generate_ecc_private_key(curve_name: str) -> EllipticCurvePrivateKeyWithSerialization:
@@ -155,7 +168,7 @@ def generate_ecc_public_key(
 def save_ecc_private_key(
     ec_private_key: EllipticCurvePrivateKeyWithSerialization,
     file_path: str,
-    password: str = None,
+    password: Optional[str] = None,
     encoding: Encoding = Encoding.PEM,
 ) -> None:
     """Save the ECC private key to the given file.
@@ -172,8 +185,7 @@ def save_ecc_private_key(
         if password
         else serialization.NoEncryption(),
     )
-    with open(file_path, "wb") as f:
-        f.write(serialized_private)
+    write_file(serialized_private, file_path, mode="wb")
 
 
 def save_ecc_public_key(
@@ -190,5 +202,32 @@ def save_ecc_public_key(
     pem_data = ec_public_key.public_bytes(
         encoding=encoding, format=serialization.PublicFormat.SubjectPublicKeyInfo
     )
-    with open(file_path, "wb") as f:
-        f.write(pem_data)
+    write_file(pem_data, file_path, mode="wb")
+
+
+def recreate_ecc_public_key(coor_x: int, coor_y: int, curve: str) -> EllipticCurvePublicKey:
+    """Recreate ECC public key from coordinates.
+
+    :param coor_x: X coordinate of point on curve.
+    :param coor_y: Y coordinate of point on curve.
+    :param curve: Name of ECC curve.
+    :return: ECC public key.
+    """
+    pub_numbers = ec.EllipticCurvePublicNumbers(
+        x=coor_x, y=coor_y, curve=get_ec_curve_object(curve)
+    )
+    key = pub_numbers.public_key()
+    return key
+
+
+def recreate_ecc_public_key_from_data(coors: bytes, curve: str) -> EllipticCurvePublicKey:
+    """Recreate ECC public key from coordinates in data blob.
+
+    :param coors: Data blob of coordinates in bytes (X,Y in Big Endian)
+    :param curve: Name of ECC curve.
+    :return: ECC public key.
+    """
+    coordinate_length = len(coors) // 2
+    coor_x = int.from_bytes(coors[:coordinate_length], byteorder="big")
+    coor_y = int.from_bytes(coors[coordinate_length:], byteorder="big")
+    return recreate_ecc_public_key(coor_x=coor_x, coor_y=coor_y, curve=curve)
