@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2020-2021 NXP
+# Copyright 2020-2023 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
@@ -11,8 +11,10 @@ from struct import pack
 from typing import Type
 
 from spsdk import SPSDKError, crypto
+from spsdk.crypto import rsa
 from spsdk.crypto.signature_provider import PlainFileSP
-from spsdk.dat import DebugAuthenticationChallenge, DebugCredential
+from spsdk.dat import DebugAuthenticationChallenge
+from spsdk.dat.debug_credential import DebugCredential
 from spsdk.dat.utils import ecc_public_numbers_to_bytes
 from spsdk.utils.crypto.backend_internal import internal_backend
 
@@ -83,13 +85,17 @@ class DebugAuthenticateResponse:
         :param data: Raw data as bytes
         :param offset: Offset of input data
         :return: DebugAuthenticateResponse object
+        :raises NotImplementedError: Derived class has to implement this method
         """
-        raise NotImplementedError
+        raise NotImplementedError("Derived class has to implement this method.")
 
     @classmethod
     def _get_class(cls, version: str, socc: int) -> "Type[DebugAuthenticateResponse]":
-        if socc == 4:
-            return _n4analog_version_mapping[version]
+        """Get the right Debug Authentication Response class by the protocol version.
+
+        :param version: DAT protocol version
+        :param socc: SOCC of used chip
+        """
         return _version_mapping[version]
 
     @classmethod
@@ -126,6 +132,7 @@ class DebugAuthenticateResponseRSA(DebugAuthenticateResponse):
         :return: signature in bytes format
         """
         key = crypto.load_private_key(file_path=self.dck_priv)
+        assert isinstance(key, rsa.RSAPrivateKey)
         key_bytes = key.private_bytes(
             encoding=crypto.Encoding.PEM,
             format=crypto.serialization.PrivateFormat.PKCS8,
@@ -137,17 +144,15 @@ class DebugAuthenticateResponseRSA(DebugAuthenticateResponse):
 class DebugAuthenticateResponseECC(DebugAuthenticateResponse):
     """Class for ECC specific of DAR."""
 
+    KEY_LENGTH = 0
+    CURVE: crypto.ec.EllipticCurve = crypto.ec.SECP256R1()
+
     def _get_signature(self) -> bytes:
         """Sign the DAR data using SignatureProvider."""
         signature = super()._get_signature()
         r, s = crypto.utils_cryptography.decode_dss_signature(signature)
-        public_numbers = crypto.EllipticCurvePublicNumbers(r, s, crypto.ec.SECP256R1())
-        signature = ecc_public_numbers_to_bytes(public_numbers=public_numbers, length=66)
-        return signature
-
-
-class DebugAuthenticateResponseN4A(DebugAuthenticateResponse):
-    """Class for N4A specific of DAR."""
+        public_numbers = crypto.EllipticCurvePublicNumbers(r, s, self.CURVE)
+        return ecc_public_numbers_to_bytes(public_numbers=public_numbers, length=self.KEY_LENGTH)
 
     def _get_common_data(self) -> bytes:
         """Collects dc, auth_beacon and UUID."""
@@ -157,37 +162,31 @@ class DebugAuthenticateResponseN4A(DebugAuthenticateResponse):
         return data
 
 
-class DebugAuthenticateResponseN4A_256(DebugAuthenticateResponseN4A):
-    """Class for N4A specific of DAR, 256 bits sized keys."""
+class DebugAuthenticateResponseECC_256(DebugAuthenticateResponseECC):
+    """Class for LPC55S3x specific of DAR, 256 bits sized keys."""
 
-    def _get_signature(self) -> bytes:
-        """Sign the DAR data using SignatureProvider."""
-        signature = super()._get_signature()
-        r, s = crypto.utils_cryptography.decode_dss_signature(signature)
-        public_numbers = crypto.EllipticCurvePublicNumbers(r, s, crypto.ec.SECP256R1())
-        return ecc_public_numbers_to_bytes(public_numbers=public_numbers, length=32)
+    KEY_LENGTH = 32
+    CURVE = crypto.ec.SECP256R1()
 
 
-class DebugAuthenticateResponseN4A_384(DebugAuthenticateResponseN4A):
-    """Class for N4A specific of DAR, 384 bits sized keys."""
+class DebugAuthenticateResponseECC_384(DebugAuthenticateResponseECC):
+    """Class for LPC55S3x specific of DAR, 384 bits sized keys."""
 
-    def _get_signature(self) -> bytes:
-        """Sign the DAR data using SignatureProvider."""
-        signature = super()._get_signature()
-        r, s = crypto.utils_cryptography.decode_dss_signature(signature)
-        public_numbers = crypto.EllipticCurvePublicNumbers(r, s, crypto.ec.SECP384R1())
-        return ecc_public_numbers_to_bytes(public_numbers=public_numbers, length=48)
+    KEY_LENGTH = 48
+    CURVE = crypto.ec.SECP384R1()
+
+
+class DebugAuthenticateResponseECC_521(DebugAuthenticateResponseECC):
+    """Class for LPC55S3x specific of DAR, 521 bits sized keys."""
+
+    KEY_LENGTH = 66
+    CURVE = crypto.ec.SECP521R1()
 
 
 _version_mapping = {
     "1.0": DebugAuthenticateResponseRSA,
     "1.1": DebugAuthenticateResponseRSA,
-    "2.0": DebugAuthenticateResponseECC,
-    "2.1": DebugAuthenticateResponseECC,
-    "2.2": DebugAuthenticateResponseECC,
-}
-
-_n4analog_version_mapping = {
-    "2.0": DebugAuthenticateResponseN4A_256,
-    "2.1": DebugAuthenticateResponseN4A_384,
+    "2.0": DebugAuthenticateResponseECC_256,
+    "2.1": DebugAuthenticateResponseECC_384,
+    "2.2": DebugAuthenticateResponseECC_521,
 }
