@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2020-2021 NXP
+# Copyright 2020-2023 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
@@ -186,7 +186,7 @@ class HabEngine(Enum):
     HAB_ENG_SW = (0xFF, "Software engine")
 
 
-def check_reserved_regions(log_addr: int, reserved_regions: list = None) -> bool:
+def check_reserved_regions(log_addr: int, reserved_regions: Optional[list] = None) -> bool:
     """Checks if the address of the log is not in conflict with CPU reserved regions.
 
     :param log_addr: address of the RAM, where we want to store hab log
@@ -256,7 +256,7 @@ def parse_hab_log(hab_sts: int, hab_cfg: int, hab_state: int, data: bytes) -> Li
     :return: list of lines to be displayed, that describes the HAB status and content of the LOG
     :raises SPSDKError: If a record has invalid data length
     """
-    result = list()
+    result = []
     result.append("=" * 60)
     result.append(f"HAB Status:  {get_hab_enum_description(HabStatus, hab_sts)}")
     result.append(f"HAB Config:  {get_hab_enum_description(HabConfig, hab_cfg)}")
