@@ -2,7 +2,7 @@
 # -*- coding: UTF-8 -*-
 #
 # Copyright 2017-2018 Martin Olejar
-# Copyright 2019-2021 NXP
+# Copyright 2019-2023 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
@@ -66,6 +66,12 @@ class BaseClass:
         """
         self._header = Header(tag=tag, param=version)
 
+    def __eq__(self, other: Any) -> bool:
+        return isinstance(other, self.__class__) and vars(other) == vars(self)
+
+    def __ne__(self, obj: Any) -> bool:
+        return not self.__eq__(obj)
+
     @property
     def version(self) -> int:
         """Format version."""
@@ -83,20 +89,27 @@ class BaseClass:
 
     @property
     def size(self) -> int:
-        """Size of the exported binary data."""
-        raise NotImplementedError()
+        """Size of the exported binary data.
+
+        :raises NotImplementedError: Derived class has to implement this method
+        """
+        raise NotImplementedError("Derived class has to implement this method.")
 
     def info(self) -> str:
-        """Description about the instance."""
-        raise NotImplementedError()
+        """Description about the instance.
+
+        :raises NotImplementedError: Derived class has to implement this method
+        """
+        raise NotImplementedError("Derived class has to implement this method.")
 
     def export(self, dbg_info: DebugInfo = DebugInfo.disabled()) -> bytes:
         """Serialization to binary form.
 
         :param dbg_info: optional instance allowing to debug exported data; provides commented export
         :return: binary representation of the instance
+        :raises NotImplementedError: Derived class has to implement this method
         """
-        raise NotImplementedError()
+        raise NotImplementedError("Derived class has to implement this method.")
 
 
 class SecretKeyBlob:
@@ -125,8 +138,9 @@ class SecretKeyBlob:
         self._data = bytearray()
 
     def __repr__(self) -> str:
-        return "SecKeyBlob <Mode: {}, Algo: {}, Flag: 0x{:02X}, Size: {}>".format(
-            self.mode, self.algorithm, self.flag, len(self._data)
+        return (
+            f"SecKeyBlob <Mode: {self.mode}, Algo: {self.algorithm}, "
+            f"Flag: 0x{self.flag:02X}, Size: {len(self._data)}>"
         )
 
     def __eq__(self, obj: Any) -> bool:
@@ -140,10 +154,10 @@ class SecretKeyBlob:
         msg = "-" * 60 + "\n"
         msg += "SecKeyBlob\n"
         msg += "-" * 60 + "\n"
-        msg += "Mode:      {}\n".format(self.mode)
-        msg += "Algorithm: {}\n".format(self.algorithm)
-        msg += "Flag:      0x{:02X}\n".format(self.flag)
-        msg += "Size:      {} Bytes\n".format(len(self._data))
+        msg += f"Mode:      {self.mode}\n"
+        msg += f"Algorithm: {self.algorithm}\n"
+        msg += f"Flag:      0x{self.flag:02X}\n"
+        msg += f"Size:      {len(self._data)} Bytes\n"
         return msg
 
     def export(self) -> bytes:
@@ -170,7 +184,7 @@ class CertificateImg(BaseClass):
         """Size of Certificate structure for bootable image."""
         return Header.SIZE + len(self._data)
 
-    def __init__(self, version: int = 0x40, data: bytes = None) -> None:
+    def __init__(self, version: int = 0x40, data: Optional[bytes] = None) -> None:
         """Initialize the certificate structure for bootable image."""
         super().__init__(SegTag.CRT, version)
         self._data = bytearray() if data is None else bytearray(data)
@@ -180,12 +194,6 @@ class CertificateImg(BaseClass):
             f"Certificate <Ver: {self.version_major}.{self.version_minor}, Size: {len(self._data)}>"
         )
 
-    def __eq__(self, obj: Any) -> bool:
-        return isinstance(obj, CertificateImg) and vars(obj) == vars(self)
-
-    def __ne__(self, obj: Any) -> bool:
-        return not self.__eq__(obj)
-
     def __len__(self) -> int:
         return len(self._data)
 
@@ -201,8 +209,9 @@ class CertificateImg(BaseClass):
     def info(self) -> str:
         """String representation of the CertificateImg."""
         msg = "-" * 60 + "\n"
-        msg += "Certificate (Ver: {:X}.{:X}, Size: {})\n".format(
-            self.version >> 4, self.version & 0xF, len(self._data)
+        msg += (
+            f"Certificate (Ver: {self.version >> 4:X}.{self.version & 0xF:X}, "
+            f"Size: {len(self._data)})\n"
         )
         msg += "-" * 60 + "\n"
         return msg
@@ -232,7 +241,7 @@ class Signature(BaseClass):
         """Size of a signature."""
         return Header.SIZE + len(self._data)
 
-    def __init__(self, version: int = 0x40, data: bytes = None) -> None:
+    def __init__(self, version: int = 0x40, data: Optional[bytes] = None) -> None:
         """Initialize the signature."""
         super().__init__(tag=SegTag.SIG, version=version)
         self._data = bytearray() if data is None else bytearray(data)
@@ -240,12 +249,6 @@ class Signature(BaseClass):
     def __repr__(self) -> str:
         return f"Signature <Ver: {self.version >> 4}.{self.version & 0xF}, Size: {len(self._data)}>"
 
-    def __eq__(self, obj: Any) -> bool:
-        return isinstance(obj, Signature) and vars(obj) == vars(self)
-
-    def __ne__(self, obj: Any) -> bool:
-        return not self.__eq__(obj)
-
     def __len__(self) -> int:
         return len(self._data)
 
@@ -261,9 +264,7 @@ class Signature(BaseClass):
     def info(self) -> str:
         """String representation of the signature."""
         msg = "-" * 60 + "\n"
-        msg += "Signature (Ver: {:X}.{:X}, Size: {})\n".format(
-            self.version >> 4, self.version & 0xF, len(self._data)
-        )
+        msg += f"Signature (Ver: {self.version >> 4:X}.{self.version & 0xF:X}, Size: {len(self._data)})\n"
         msg += "-" * 60 + "\n"
         return msg
 
@@ -386,26 +387,21 @@ class MAC(BaseClass):
         self.data = nonce + mac
 
     def __repr__(self) -> str:
-        return "MAC <Ver: {:X}.{:X}, Nonce: {}, MAC: {}>".format(
-            self.version_major, self.version_minor, self.nonce_len, self.mac_len
+        return (
+            f"MAC <Ver: {self.version_major:X}.{self.version_minor:X}, "
+            f"Nonce: {self.nonce_len}, MAC: {self.mac_len}>"
         )
 
-    def __eq__(self, obj: Any) -> bool:
-        return isinstance(obj, MAC) and vars(obj) == vars(self)
-
-    def __ne__(self, obj: Any) -> bool:
-        return not self.__eq__(obj)
-
     def __len__(self) -> int:
         return len(self._data)
 
     def info(self) -> str:
         """Text info about the instance."""
         msg = "-" * 60 + "\n"
-        msg += "MAC (Version: {:X}.{:X})\n".format(self.version >> 4, self.version & 0xF)
+        msg += f"MAC (Version: {self.version >> 4:X}.{self.version & 0xF:X})\n"
         msg += "-" * 60 + "\n"
-        msg += "Nonce Len: {} Bytes\n".format(self.nonce_len)
-        msg += "MAC Len:   {} Bytes\n".format(self.mac_len)
+        msg += f"Nonce Len: {self.nonce_len} Bytes\n"
+        msg += f"MAC Len:   {self.mac_len} Bytes\n"
         msg += f"[{self._data.hex()}]\n"
         return msg
 
@@ -468,30 +464,49 @@ class SrkItem:
     is an algorithm identifier, not a version number.
     """
 
+    def __eq__(self, other: Any) -> bool:
+        return isinstance(other, self.__class__) and vars(other) == vars(self)
+
+    def __ne__(self, obj: Any) -> bool:
+        return not self.__eq__(obj)
+
     @property
     def size(self) -> int:
-        """Size of the exported binary data."""
-        raise NotImplementedError()
+        """Size of the exported binary data.
+
+        :raises NotImplementedError: Derived class has to implement this method
+        """
+        raise NotImplementedError("Derived class has to implement this method.")
 
     def info(self) -> str:
-        """Description about the instance."""
-        raise NotImplementedError()
+        """Description about the instance.
+
+        :raises NotImplementedError: Derived class has to implement this method
+        """
+        raise NotImplementedError("Derived class has to implement this method.")
 
     def sha256(self) -> bytes:
-        """Export SHA256 hash of the original data."""
-        raise NotImplementedError()
+        """Export SHA256 hash of the original data.
+
+        :raises NotImplementedError: Derived class has to implement this method
+        """
+        raise NotImplementedError("Derived class has to implement this method.")
 
     def hashed_entry(self) -> "SrkItem":
-        """This SRK item should be replaced with an incomplete entry with its digest."""
-        raise NotImplementedError()
+        """This SRK item should be replaced with an incomplete entry with its digest.
+
+        :raises NotImplementedError: Derived class has to implement this method
+        """
+        raise NotImplementedError("Derived class has to implement this method.")
 
     def export(self, dbg_info: DebugInfo = DebugInfo.disabled()) -> bytes:
         """Serialization to binary form.
 
         :param dbg_info: optional instance allowing to debug exported data; provides commented export
         :return: binary representation of the instance
+        :raises NotImplementedError: Derived class has to implement this method
         """
-        raise NotImplementedError()
+        raise NotImplementedError("Derived class has to implement this method.")
 
     @classmethod
     def parse(cls, data: bytes, offset: int = 0) -> "SrkItem":
@@ -554,18 +569,12 @@ class SrkItemHash(SrkItem):
         self._header.length += len(digest)
 
     def __repr__(self) -> str:
-        return "SRK Hash <Algorithm: {}>".format(EnumAlgorithm[self._header.param])  # type: ignore
-
-    def __eq__(self, obj: Any) -> bool:
-        return isinstance(obj, SrkItemHash) and vars(obj) == vars(self)
-
-    def __ne__(self, obj: Any) -> bool:
-        return not self.__eq__(obj)
+        return f"SRK Hash <Algorithm: {EnumAlgorithm[self._header.param]}>"  # type: ignore
 
     def info(self) -> str:
         """String representation of SrkItemHash."""
         msg = str()
-        msg += "Hash algorithm: {}\n".format(EnumAlgorithm[self._header.param])  # type: ignore
+        msg += f"Hash algorithm: {EnumAlgorithm[self._header.param]}\n"  # type: ignore
         msg += "Hash value:\n"
         msg += hexdump_fmt(self.digest)
         return msg
@@ -641,28 +650,22 @@ class SrkItemRSA(SrkItem):
         self._header.length += 8 + len(self.modulus) + len(self.exponent)
 
     def __repr__(self) -> str:
-        return "SRK <Algorithm: {}, CA: {}>".format(
-            EnumAlgorithm[self.algorithm],  # type: ignore
-            "YES" if self.flag == 0x80 else "NO",
-        )  # type: ignore
-
-    def __eq__(self, obj: Any) -> bool:
-        return isinstance(obj, SrkItemRSA) and vars(obj) == vars(self)
-
-    def __ne__(self, obj: Any) -> bool:
-        return not self.__eq__(obj)
+        return (
+            f"SRK <Algorithm: {EnumAlgorithm[self.algorithm]}, "  # type: ignore
+            f"CA: {'YES' if self.flag == 0x80 else 'NO'}>"
+        )
 
     def info(self) -> str:
         """String representation of SrkItemRSA."""
-        msg = str()
-        msg += "Algorithm: {}\n".format(EnumAlgorithm[self.algorithm])  # type: ignore
-        msg += "Flag:      0x{:02X} {}\n".format(self.flag, "(CA)" if self.flag == 0x80 else "")
-        msg += "Length:    {} bit\n".format(self.key_length)
-        msg += "Modulus:\n"
-        msg += modulus_fmt(self.modulus)
-        msg += "\n"
-        msg += "Exponent: {0} (0x{0:X})\n".format(int.from_bytes(self.exponent, "big"))
-        return msg
+        exp = int.from_bytes(self.exponent, "big")
+        return (
+            f"Algorithm: {EnumAlgorithm[self.algorithm]}\n"  # type: ignore
+            f"Flag:      0x{self.flag:02X} {'(CA)' if self.flag == 0x80 else ''}\n"
+            f"Length:    {self.key_length} bit\n"
+            "Modulus:\n"
+            f"{modulus_fmt(self.modulus)}\n"
+            f"Exponent: {exp} (0x{exp:X})\n"
+        )
 
     def sha256(self) -> bytes:
         """Export SHA256 hash of the data."""
@@ -705,7 +708,7 @@ class SrkItemRSA(SrkItem):
 
         flag = 0
         try:
-            key_usage = cert.extensions.get_extension_for_class(KeyUsage)
+            key_usage = cert.extensions.get_extension_for_class(KeyUsage)  # type:ignore
             assert isinstance(key_usage.value, KeyUsage)
             if key_usage.value.key_cert_sign:
                 flag = 0x80
@@ -747,16 +750,11 @@ class SrkTable(BaseClass):
         self._keys: List[SrkItem] = []
 
     def __repr__(self) -> str:
-        return "SRK_Table <Version: {:X}.{:X}, Keys: {}>".format(
-            self.version_major, self.version_minor, len(self._keys)
+        return (
+            f"SRK_Table <Version: {self.version_major:X}.{self.version_minor:X},"
+            f" Keys: {len(self._keys)}>"
         )
 
-    def __eq__(self, obj: Any) -> bool:
-        return isinstance(obj, SrkTable) and vars(obj) == vars(self)
-
-    def __ne__(self, obj: Any) -> bool:
-        return not self.__eq__(obj)
-
     def __len__(self) -> int:
         return len(self._keys)
 
@@ -773,8 +771,9 @@ class SrkTable(BaseClass):
     def info(self) -> str:
         """Text info about the instance."""
         msg = "-" * 60 + "\n"
-        msg += "SRK Table (Version: {:X}.{:X}, #Keys: {})\n".format(
-            self.version_major, self.version_minor, len(self._keys)
+        msg += (
+            f"SRK Table (Version: {self.version_major:X}.{self.version_minor:X}, "
+            f"#Keys: {len(self._keys)})\n"
         )
         msg += "-" * 60 + "\n"
         for i, srk in enumerate(self._keys):
