@@ -2,7 +2,7 @@
 # -*- coding: UTF-8 -*-
 #
 # Copyright 2016-2018 Martin Olejar
-# Copyright 2019-2021 NXP
+# Copyright 2019-2023 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
@@ -10,23 +10,27 @@
 
 import logging
 import struct
-from typing import List, Optional, Tuple, Union
+import time
+from contextlib import contextmanager
+from typing import Generator, List, NamedTuple, Optional, Tuple, Union
 
-import construct
 from crcmod.predefined import mkPredefinedCrcFun
-from serial import Serial
+from serial import Serial, SerialTimeoutException
 from serial.tools.list_ports import comports
 
 from spsdk.mboot.commands import CmdPacket, CmdResponse, parse_cmd_response
 from spsdk.mboot.exceptions import McuBootConnectionError, McuBootDataAbortError
 from spsdk.utils.easy_enum import Enum
+from spsdk.utils.misc import Timeout
 
-from .base import Interface
+from .base import MBootInterface
 
 logger = logging.getLogger(__name__)
 
 
-def scan_uart(port: str = None, baudrate: int = None, timeout: int = None) -> List[Interface]:
+def scan_uart(
+    port: Optional[str] = None, baudrate: Optional[int] = None, timeout: Optional[int] = None
+) -> List["Uart"]:
     """Scan connected serial ports.
 
     Returns list of serial ports with devices that respond to PING command.
@@ -49,7 +53,7 @@ def scan_uart(port: str = None, baudrate: int = None, timeout: int = None) -> Li
     return list(filter(None, all_ports))
 
 
-def _check_port(port: str, baudrate: int, timeout: int) -> Optional[Interface]:
+def _check_port(port: str, baudrate: int, timeout: int) -> Optional["Uart"]:
     """Check if device on comport 'port' responds to PING command.
 
     :param port: name of port to check
@@ -86,25 +90,35 @@ def to_int(data: bytes, little_endian: bool = True) -> int:
     :return: integer
     """
     byte_order = "little" if little_endian else "big"
-    return int.from_bytes(data, byteorder=byte_order)
+    return int.from_bytes(data, byteorder=byte_order)  # type: ignore[arg-type]
 
 
-#: Version of protocol used in serial communication
-PROTOCOL_VERSION = construct.Struct(
-    "bugfix" / construct.Int8ul,
-    "minor" / construct.Int8ul,
-    "major" / construct.Int8ul,
-    "name" / construct.Int8ul,
-)
+class PingResponse(NamedTuple):
+    """Special type of response for Ping Command."""
 
-#: Type of frame used for pig response
-PING_RESPONSE = construct.Struct(
-    "version" / PROTOCOL_VERSION,
-    "options" / construct.Int16ul,
-    "crc" / construct.Int16ul,
-)
+    version: int
+    options: int
+    crc: int
 
+    @classmethod
+    def parse(cls, data: bytes) -> "PingResponse":
+        """Parse raw data into PingResponse object.
+
+        :param data: bytes to be unpacked to PingResponse object
+            4B version, 2B data, 2B CRC16
+        :raises McuBootConnectionError: Received invalid ping response
+        :return: PingResponse
+        """
+        try:
+            version, options, crc = struct.unpack("<I2H", data)
+        except struct.error as err:
+            raise McuBootConnectionError("Received invalid ping response") from err
+        return cls(version, options, crc)
+
+
+PING_TIMEOUT_MS = 500
 MAX_PING_RESPONSE_DUMMY_BYTES = 50
+MAX_UART_OPEN_ATTEMPTS = 3
 
 ########################################################################################################################
 # UART Interface Class
@@ -121,30 +135,38 @@ class FPType(Enum):
     PINGR = 0xA7
 
 
-class Uart(Interface):
+class Uart(MBootInterface):
     """UART interface."""
 
     FRAME_START_BYTE = 0x5A
+    FRAME_START_BYTE_NOT_READY = 0x00
 
     @property
     def is_opened(self) -> bool:
-        """Return True if device is open, False othervise."""
+        """Return True if device is open, False otherwise."""
         return self.device.is_open
 
-    def __init__(self, port: str = None, baudrate: int = 57600, timeout: int = 5000) -> None:
+    def __init__(
+        self, port: Optional[str] = None, baudrate: int = 57600, timeout: int = 5000
+    ) -> None:
         """Initialize the UART interface.
 
         :param port: name of the serial port, defaults to None
         :param baudrate: baudrate of the serial port, defaults to 57600
-        :param timeout: read/write timeout in milliseconds, defaults to 2000
+        :param timeout: read/write timeout in milliseconds, defaults to 5000
         :raises McuBootConnectionError: when the port could not be opened
         """
         super().__init__()
         try:
-            self.device = Serial(port=port, timeout=timeout / 1000, baudrate=baudrate)
-            self.close()
-            self.protocol_version = None
-            self.options = None
+            self.timeout = timeout
+            timeout_s = timeout / 1000
+            self.device = Serial(
+                port=port, timeout=timeout_s, write_timeout=timeout_s, baudrate=baudrate
+            )
+            if port:
+                self.close()  # TODO: [SPSDK-1212] Is this really necessary here? Please advice somebody.
+            self.protocol_version = 0
+            self.options = 0
         except Exception as e:
             raise McuBootConnectionError(str(e)) from e
 
@@ -153,12 +175,27 @@ class Uart(Interface):
 
         :raises McuBootConnectionError: In any case of fail of UART open operation.
         """
-        try:
-            self.device.open()
-            self.ping()
-        except Exception as exc:
-            self.device.close()
-            raise McuBootConnectionError("UART Interface open operation fails.") from exc
+        for i in range(MAX_UART_OPEN_ATTEMPTS):
+            try:
+                self.device.open()
+                self.ping()
+                logger.debug(f"Interface opened after {i + 1} attempts.")
+                return
+            except TimeoutError as e:
+                self.device.reset_input_buffer()
+                self.device.reset_output_buffer()
+                # Closing may take up 30-40 seconds
+                self.device.close()
+                logger.debug(f"Timeout when pinging the device: {repr(e)}")
+            except McuBootConnectionError as e:
+                self.device.close()
+                logger.debug(f"Opening interface failed with: {repr(e)}")
+            except Exception as exc:
+                self.device.close()
+                raise McuBootConnectionError("UART Interface open operation fails.") from exc
+        raise McuBootConnectionError(
+            f"Cannot open UART interface after {MAX_UART_OPEN_ATTEMPTS} attempts."
+        )
 
     def close(self) -> None:
         """Close the UART interface.
@@ -189,12 +226,12 @@ class Uart(Interface):
         :raises McuBootConnectionError: When received invalid CRC
         """
         _, frame_type = self._read_frame_header()
-        length = to_int(self._read(2))
-        crc = to_int(self._read(2))
+        length = to_int(self._read_default(2))
+        crc = to_int(self._read_default(2))
         if not length:
             self._send_ack()
             raise McuBootDataAbortError()
-        data = self._read(length)
+        data = self._read_default(length)
         self._send_ack()
         calculated_crc = self._calc_frame_crc(data, frame_type)
         if crc != calculated_crc:
@@ -207,16 +244,29 @@ class Uart(Interface):
         """Write data to the device; data might be in form of 'CmdPacket' or bytes.
 
         :param packet: Packet to send
+        :raises AttributeError: frame type is incorrect
         """
+        data, frame_type = None, None
         if isinstance(packet, CmdPacket):
             data = packet.to_bytes(padding=False)
             frame_type = FPType.CMD
         if isinstance(packet, (bytes, bytearray)):
             data = packet
             frame_type = FPType.DATA
+        if not data or not frame_type:
+            raise AttributeError("Incorrect packet type")
         frame = self._create_frame(data, frame_type)
         self._send_frame(frame, wait_for_ack=True)
 
+    def _read_default(self, length: int) -> bytes:
+        """Read 'length' amount of bytes from device, this function can be overridden in child class.
+
+        :param length: Number of bytes to read
+        :type length: int
+        :return: Data read from the device
+        """
+        return self._read(length)
+
     def _read(self, length: int) -> bytes:
         """Read 'length' amount for bytes from device.
 
@@ -238,6 +288,7 @@ class Uart(Interface):
         """Send data to device.
 
         :param data: Data to send
+        :raises TimeoutError: Time-out
         :raises McuBootConnectionError: When sending the data fails
         """
         logger.debug(f"[{' '.join(f'{b:02x}' for b in data)}]")
@@ -246,6 +297,10 @@ class Uart(Interface):
             self.device.reset_output_buffer()
             self.device.write(data)
             self.device.flush()
+        except SerialTimeoutException as e:
+            raise TimeoutError(
+                f"Write timeout error. The timeout is set to {self.device.write_timeout} s. Consider increasing it."
+            ) from e
         except Exception as e:
             raise McuBootConnectionError(str(e)) from e
 
@@ -263,7 +318,7 @@ class Uart(Interface):
         if wait_for_ack:
             self._read_frame_header(FPType.ACK)
 
-    def _read_frame_header(self, expected_frame_type: int = None) -> Tuple[int, int]:
+    def _read_frame_header(self, expected_frame_type: Optional[int] = None) -> Tuple[int, int]:
         """Read frame header and frame type. Return them as tuple of integers.
 
         :param expected_frame_type: Check if the frame_type is exactly as expected
@@ -272,15 +327,28 @@ class Uart(Interface):
         :raises McuBootConnectionError: Unexpected frame header or frame type (if specified)
         :raises McuBootConnectionError: When received invalid ACK
         """
-        header = to_int(self._read(1))
-        if header != self.FRAME_START_BYTE:
+        timeout = Timeout(self.timeout, "ms")
+        header = -1
+        while not timeout.overflow():
+            header = to_int(self._read_default(1))
+            if header != self.FRAME_START_BYTE_NOT_READY:
+                break
+        # This is workaround addressing SPI ISP issue on RT5/6xx when sometimes
+        # ACK frames and START BYTE frames are swapped, see SPSDK-1824 for more details
+        if header not in [self.FRAME_START_BYTE, FPType.ACK]:
             raise McuBootConnectionError(
                 f"Received invalid frame header '{header:#X}' expected '{self.FRAME_START_BYTE:#X}'"
+                + "\nTry increasing the timeout, some operations might take longer"
             )
-        frame_type = to_int(self._read(1))
+        if header == FPType.ACK:
+            frame_type = header
+        else:
+            frame_type = to_int(self._read(1))
         if frame_type == FPType.ABORT:
             raise McuBootDataAbortError()
         if expected_frame_type:
+            if frame_type == self.FRAME_START_BYTE:
+                frame_type = header
             if frame_type != expected_frame_type:
                 raise McuBootConnectionError(
                     f"received invalid ACK '{frame_type:#X}' expected '{expected_frame_type:#X}'"
@@ -322,44 +390,68 @@ class Uart(Interface):
         :raises McuBootConnectionError: If the ping response is not received
         :raises McuBootConnectionError: If crc does not match
         """
-        ping = struct.pack("<BB", self.FRAME_START_BYTE, FPType.PING)
-        self._send_frame(ping, wait_for_ack=False)
+        with self.ping_timeout(timeout=PING_TIMEOUT_MS):
+            ping = struct.pack("<BB", self.FRAME_START_BYTE, FPType.PING)
+            self._send_frame(ping, wait_for_ack=False)
 
-        # after power cycle, MBoot v 3.0+ may respond to first command with a leading dummy data
-        # we read data from UART until the FRAME_START_BYTE byte
-        start_byte = b""
-        for i in range(MAX_PING_RESPONSE_DUMMY_BYTES):
-            start_byte = self._read(1)
-            if start_byte is None:
-                raise McuBootConnectionError("Failed to receive initial byte")
+            # after power cycle, MBoot v 3.0+ may respond to first command with a leading dummy data
+            # we read data from UART until the FRAME_START_BYTE byte
+            start_byte = b""
+            for i in range(MAX_PING_RESPONSE_DUMMY_BYTES):
+                start_byte = self._read_default(1)
+                if start_byte is None:
+                    raise McuBootConnectionError("Failed to receive initial byte")
 
-            if start_byte == self.FRAME_START_BYTE.to_bytes(length=1, byteorder="little"):
-                logger.debug(f"FRAME_START_BYTE received in {i + 1}. attempt.")
-                break
-        else:
-            raise McuBootConnectionError("Failed to receive FRAME_START_BYTE")
+                if start_byte == self.FRAME_START_BYTE.to_bytes(length=1, byteorder="little"):
+                    logger.debug(f"FRAME_START_BYTE received in {i + 1}. attempt.")
+                    break
+            else:
+                raise McuBootConnectionError("Failed to receive FRAME_START_BYTE")
 
-        header = to_int(start_byte)
-        if header != self.FRAME_START_BYTE:
-            raise McuBootConnectionError("Header is invalid")
-        frame_type = to_int(self._read(1))
-        if frame_type != FPType.PINGR:
-            raise McuBootConnectionError("Frame type is invalid")
+            header = to_int(start_byte)
+            if header != self.FRAME_START_BYTE:
+                raise McuBootConnectionError("Header is invalid")
+            frame_type = to_int(self._read_default(1))
+            if frame_type != FPType.PINGR:
+                raise McuBootConnectionError("Frame type is invalid")
 
-        response_data = self._read(8)
-        if response_data is None:
-            raise McuBootConnectionError("Failed to receive ping response")
-        response = PING_RESPONSE.parse(response_data)
+            response_data = self._read_default(8)
+            if response_data is None:
+                raise McuBootConnectionError("Failed to receive ping response")
+            response = PingResponse.parse(response_data)
 
-        # ping response has different crc computation than the other responses
-        # that's why we can't use calc_frame_crc method
-        # crc data for ping excludes the last 2B of response data, which holds the CRC from device
-        crc_data = struct.pack(
-            f"<BB{len(response_data) -2}B", header, frame_type, *response_data[:-2]
-        )
-        crc = calc_crc(crc_data)
-        if crc != response.crc:
-            raise McuBootConnectionError("Received CRC doesn't match")
+            # ping response has different crc computation than the other responses
+            # that's why we can't use calc_frame_crc method
+            # crc data for ping excludes the last 2B of response data, which holds the CRC from device
+            crc_data = struct.pack(
+                f"<BB{len(response_data) -2}B", header, frame_type, *response_data[:-2]
+            )
+            crc = calc_crc(crc_data)
+            if crc != response.crc:
+                raise McuBootConnectionError("Received CRC doesn't match")
 
-        self.protocol_version = response.version
-        self.options = response.options
+            self.protocol_version = response.version
+            self.options = response.options
+
+    @contextmanager
+    def ping_timeout(self, timeout: int = PING_TIMEOUT_MS) -> Generator[None, None, None]:
+        """Context manager for changing UART's timeout.
+
+        :param timeout: New temporary timeout in milliseconds, defaults to PING_TIMEOUT_MS (500ms)
+        :return: Generator[None, None, None]
+        """
+        context_timeout = min(timeout, self.timeout)
+        context_timeout_s = context_timeout / 1000
+        self.device.timeout = context_timeout_s
+        self.device.write_timeout = context_timeout_s
+        logger.debug(f"Setting timeout to {context_timeout} ms")
+        # driver needs to be reconfigured after timeout change, wait for a little while
+        time.sleep(0.005)
+
+        yield
+
+        restored_timeout_s = self.timeout / 1000
+        self.device.timeout = restored_timeout_s
+        self.device.write_timeout = restored_timeout_s
+        logger.debug(f"Restoring timeout to {self.timeout} ms")
+        time.sleep(0.005)
