@@ -1,13 +1,14 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright (c) 2019-2022 NXP
+# Copyright (c) 2019-2023 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
 """Module for USB-SIO communication with a target device using MBoot protocol."""
 
 import logging
+import re
 from typing import List, Optional
 
 import libusbsio
@@ -15,9 +16,9 @@ from libusbsio.libusbsio import LIBUSBSIO
 
 from spsdk import SPSDKError, SPSDKValueError
 from spsdk.mboot.exceptions import McuBootConnectionError
+from spsdk.utils.misc import value_to_int
 from spsdk.utils.usbfilter import USBDeviceFilter
 
-from .base import MBootInterface
 from .uart import Uart
 
 logger = logging.getLogger(__name__)
@@ -31,14 +32,15 @@ def _get_usbsio() -> LIBUSBSIO:
     """
     try:
         # get the global singleton instance of LIBUSBSIO library
-        return libusbsio.usbsio(loglevel=logger.level)
+        libusbsio_logger = logging.getLogger("libusbsio")
+        return libusbsio.usbsio(loglevel=libusbsio_logger.getEffectiveLevel())
     except libusbsio.LIBUSBSIO_Exception as e:
         raise SPSDKError(f"Error in libusbsio interface: {e}") from e
     except Exception as e:
         raise SPSDKError(str(e)) from e
 
 
-def get_usbsio_devices(config: str = None) -> List[int]:
+def get_usbsio_devices(config: Optional[str] = None) -> List[int]:
     """Returns list of ports indexes of USBSIO devices.
 
     It could be filtered by standard SPSDK USB filters.
@@ -90,7 +92,7 @@ def get_usbsio_devices(config: str = None) -> List[int]:
     return port_indexes
 
 
-def scan_usbsio(config: str = None, timeout: int = 5000) -> List["UsbSio"]:
+def scan_usbsio(config: Optional[str] = None, timeout: int = 5000) -> List["UsbSio"]:
     """Scan connected USB-SIO bridge devices.
 
     :param config: Configuration string identifying spi or i2c SIO interface
@@ -101,9 +103,20 @@ def scan_usbsio(config: str = None, timeout: int = 5000) -> List["UsbSio"]:
     :raises SPSDKValueError: Invalid configuration detected.
     """
     cfg = config.split(",") if config else []
-    if all(intf in cfg for intf in ["i2c", "spi"]):
+    re_spi = re.compile(r"^spi(?P<index>\d*)")
+    re_i2c = re.compile(r"^i2c(?P<index>\d*)")
+    spi = None
+    i2c = None
+    for cfg_part in cfg:
+        match_i2c = re_i2c.match(cfg_part.lower())
+        if match_i2c:
+            i2c = value_to_int(match_i2c.group("index"), 0)
+        match_spi = re_spi.match(cfg_part.lower())
+        if match_spi:
+            spi = value_to_int(match_spi.group("index"), 0)
+    if i2c is not None and spi is not None:
         raise SPSDKValueError(f"Cannot be specified spi and i2c together in configuration: {cfg}")
-    intf_specified = any(intf in cfg for intf in ["i2c", "spi"])
+    intf_specified = i2c is not None or spi is not None
 
     port_indexes = get_usbsio_devices(config)
     sio = _get_usbsio()
@@ -113,16 +126,16 @@ def scan_usbsio(config: str = None, timeout: int = 5000) -> List["UsbSio"]:
             raise SPSDKError(f"Cannot open libusbsio bridge {port}.")
         i2c_ports = sio.GetNumI2CPorts()
         if i2c_ports:
-            if "i2c" in cfg:
-                devices.append(UsbSioI2C(dev=port, config=config, timeout=timeout))
+            if i2c is not None:
+                devices.append(UsbSioI2C(dev=port, port=i2c, config=config, timeout=timeout))
             elif not intf_specified:
                 devices.extend(
                     [UsbSioI2C(dev=port, port=p, timeout=timeout) for p in range(i2c_ports)]
                 )
         spi_ports = sio.GetNumSPIPorts()
         if spi_ports:
-            if "spi" in cfg:
-                devices.append(UsbSioSPI(dev=port, config=config, timeout=timeout))
+            if spi is not None:
+                devices.append(UsbSioSPI(dev=port, port=spi, config=config, timeout=timeout))
             elif not intf_specified:
                 devices.extend(
                     [UsbSioSPI(dev=port, port=p, timeout=timeout) for p in range(spi_ports)]
@@ -134,7 +147,7 @@ def scan_usbsio(config: str = None, timeout: int = 5000) -> List["UsbSio"]:
     return devices
 
 
-def scan_usbsio_i2c(config: str = None, timeout: int = 5000) -> List["UsbSioI2C"]:
+def scan_usbsio_i2c(config: Optional[str] = None, timeout: int = 5000) -> List["UsbSioI2C"]:
     """Scan connected USB-SIO bridge devices and return just I2C devices.
 
     :param config: Configuration string identifying spi or i2c SIO interface
@@ -146,7 +159,7 @@ def scan_usbsio_i2c(config: str = None, timeout: int = 5000) -> List["UsbSioI2C"
     return [x for x in devices if isinstance(x, UsbSioI2C)]
 
 
-def scan_usbsio_spi(config: str = None, timeout: int = 5000) -> List["UsbSioSPI"]:
+def scan_usbsio_spi(config: Optional[str] = None, timeout: int = 5000) -> List["UsbSioSPI"]:
     """Scan connected USB-SIO bridge devices and return just SPI devices.
 
     :param config: Configuration string identifying spi or i2c SIO interface
@@ -171,7 +184,7 @@ class UsbSio(Uart):
         """Indicates whether interface is open."""
         return bool(self.port)
 
-    def __init__(self, dev: int = 0, config: str = None, timeout: int = 5000) -> None:
+    def __init__(self, dev: int = 0, config: Optional[str] = None, timeout: int = 5000) -> None:
         """Initialize the Interface object.
 
         :param dev: device index to be used, default is set to 0
@@ -193,7 +206,9 @@ class UsbSio(Uart):
 
     def info(self) -> str:
         """Return string containing information about the interface."""
-        return f"libusbsio interface '{self.config}'"
+        class_name = self.__class__.__name__
+        config = f":'{self.config}'" if self.config else ""
+        return f"libusbsio interface ({class_name}){config}"
 
     @staticmethod
     def get_interface_cfg(config: str, interface: str) -> str:
@@ -216,7 +231,7 @@ class UsbSioSPI(UsbSio):
 
     def __init__(
         self,
-        config: str = None,
+        config: Optional[str] = None,
         dev: int = 0,
         port: int = 0,
         ssel_port: int = 0,
@@ -335,7 +350,7 @@ class UsbSioI2C(UsbSio):
 
     def __init__(
         self,
-        config: str = None,
+        config: Optional[str] = None,
         dev: int = 0,
         port: int = 0,
         address: int = 0x10,
@@ -389,6 +404,8 @@ class UsbSioI2C(UsbSio):
     def close(self) -> None:
         """Close the interface."""
         if self.port:
+            # Preventive Reset of I2C device to be ready to reopen in case of any pending problem
+            self.port.Reset()
             self.port.Close()
             self.port = None
             # if not self.sio.IsAnyPortOpen(): #TODO temporary hack till new version of libusbsio greater than 2.1.11
