@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2019-2022 NXP
+# Copyright 2019-2023 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
@@ -17,10 +17,17 @@ from spsdk import SPSDKError
 from spsdk.mboot import ExtMemId, MemId
 from spsdk.sbfile.misc import SecBootBlckSize
 from spsdk.utils.crypto.abstract import BaseClass
-from spsdk.utils.crypto.common import swap16
 from spsdk.utils.easy_enum import Enum
-from spsdk.utils.misc import DebugInfo
+from spsdk.utils.misc import DebugInfo, swap16
 
+########################################################################################################################
+# Constants
+########################################################################################################################
+
+DEVICE_ID_MASK = 0xFF
+DEVICE_ID_SHIFT = 0
+GROUP_ID_MASK = 0xF00
+GROUP_ID_SHIFT = 8
 
 ########################################################################################################################
 # Enums
@@ -131,11 +138,19 @@ class CmdHeader:
 class CmdBaseClass(BaseClass):
     """Base class for all commands."""
 
+    # bit mask for device ID inside flags
+    ROM_MEM_DEVICE_ID_MASK = 0xFF00
+    # shift for device ID inside flags
+    ROM_MEM_DEVICE_ID_SHIFT = 8
+    # bit mask for group ID inside flags
+    ROM_MEM_GROUP_ID_MASK = 0xF0
+    # shift for group ID inside flags
+    ROM_MEM_GROUP_ID_SHIFT = 4
+
     def __init__(self, tag: int) -> None:
         """Initialize CmdBase."""
         self._header = CmdHeader(tag)
 
-    # TODO Refactor: so header is not published, it should not be accessed publicly
     @property
     def header(self) -> CmdHeader:
         """Return command header."""
@@ -151,7 +166,7 @@ class CmdBaseClass(BaseClass):
 
     def info(self) -> str:
         """Return text info about the instance."""
-        return self.__str__() + "\n"  # default implementation is same as __str__
+        return str(self) + "\n"  # default implementation is same as __str__
 
     def export(self, dbg_info: DebugInfo = DebugInfo.disabled()) -> bytes:
         """Return object serialized into bytes."""
@@ -234,6 +249,16 @@ class CmdLoad(CmdBaseClass):
             raise SPSDKError("Incorrect address")
         self._header.address = value
 
+    @property
+    def flags(self) -> int:
+        """Return command's flag."""
+        return self._header.flags
+
+    @flags.setter
+    def flags(self, value: int) -> None:
+        """Set command's flag."""
+        self._header.flags = value
+
     @property
     def raw_size(self) -> int:
         """Return aligned size of the command including header and data."""
@@ -242,15 +267,30 @@ class CmdLoad(CmdBaseClass):
             size += CmdHeader.SIZE - (size % CmdHeader.SIZE)
         return size
 
-    def __init__(self, address: int, data: bytes) -> None:
+    def __init__(self, address: int, data: bytes, mem_id: int = 0) -> None:
         """Initialize CMD Load."""
         super().__init__(EnumCmdTag.LOAD)
         assert isinstance(data, (bytes, bytearray))
         self.address = address
         self.data = bytes(data)
+        self.mem_id = mem_id
+
+        device_id = get_device_id(mem_id)
+        group_id = get_group_id(mem_id)
+
+        self.flags |= (self.flags & ~self.ROM_MEM_DEVICE_ID_MASK) | (
+            (device_id << self.ROM_MEM_DEVICE_ID_SHIFT) & self.ROM_MEM_DEVICE_ID_MASK
+        )
+
+        self.flags |= (self.flags & ~self.ROM_MEM_GROUP_ID_MASK) | (
+            (group_id << self.ROM_MEM_GROUP_ID_SHIFT) & self.ROM_MEM_GROUP_ID_MASK
+        )
 
     def __str__(self) -> str:
-        return f"LOAD: Address=0x{self.address:08X}, DataLen={len(self.data)}"
+        return (
+            f"LOAD: Address=0x{self.address:08X}, DataLen={len(self.data)}, "
+            f"Flags=0x{self.flags:08X}, MemId=0x{self.mem_id:08X}"
+        )
 
     def export(self, dbg_info: DebugInfo = DebugInfo.disabled()) -> bytes:
         """Export command as binary."""
@@ -287,7 +327,10 @@ class CmdLoad(CmdBaseClass):
         crc32_function = mkPredefinedCrcFun("crc-32-mpeg")
         if header.data != crc32_function(cmd_data, 0xFFFFFFFF):
             raise SPSDKError("Invalid CRC in the command header")
-        obj = CmdLoad(header.address, cmd_data)
+        device_id = (header.flags & cls.ROM_MEM_DEVICE_ID_MASK) >> cls.ROM_MEM_DEVICE_ID_SHIFT
+        group_id = (header.flags & cls.ROM_MEM_GROUP_ID_MASK) >> cls.ROM_MEM_GROUP_ID_SHIFT
+        mem_id = get_memory_id(device_id, group_id)
+        obj = CmdLoad(header.address, cmd_data, mem_id)
         obj.header.data = header.data
         obj.header.flags = header.flags
         obj._update_data()
@@ -413,8 +456,6 @@ class CmdJump(CmdBaseClass):
     @argument.setter
     def argument(self, value: int) -> None:
         """Set command's argument."""
-        if value < 0x00 or value > 0xFF:
-            raise SPSDKError("Incorrect argument")
         self._header.data = value
 
     @property
@@ -551,16 +592,29 @@ class CmdErase(CmdBaseClass):
         """Set command's flag."""
         self._header.flags = value
 
-    def __init__(self, address: int = 0, length: int = 0, flags: int = 0) -> None:
+    def __init__(self, address: int = 0, length: int = 0, flags: int = 0, mem_id: int = 0) -> None:
         """Initialize Command Erase."""
         super().__init__(EnumCmdTag.ERASE)
         self.address = address
         self.length = length
         self.flags = flags
+        self.mem_id = mem_id
+
+        device_id = get_device_id(mem_id)
+        group_id = get_group_id(mem_id)
+
+        self.flags |= (self.flags & ~self.ROM_MEM_DEVICE_ID_MASK) | (
+            (device_id << self.ROM_MEM_DEVICE_ID_SHIFT) & self.ROM_MEM_DEVICE_ID_MASK
+        )
+
+        self.flags |= (self.flags & ~self.ROM_MEM_GROUP_ID_MASK) | (
+            (group_id << self.ROM_MEM_GROUP_ID_SHIFT) & self.ROM_MEM_GROUP_ID_MASK
+        )
 
     def __str__(self) -> str:
         return (
-            f"ERASE: Address=0x{self.address:08X}, Length={self.length}, Flags=0x{self.flags:08X}"
+            f"ERASE: Address=0x{self.address:08X}, Length={self.length}, Flags=0x{self.flags:08X}, "
+            f"MemId=0x{self.mem_id:08X}"
         )
 
     @classmethod
@@ -575,7 +629,10 @@ class CmdErase(CmdBaseClass):
         header = CmdHeader.parse(data, offset)
         if header.tag != EnumCmdTag.ERASE:
             raise SPSDKError("Invalid header tag")
-        return cls(header.address, header.count, header.flags)
+        device_id = (header.flags & cls.ROM_MEM_DEVICE_ID_MASK) >> cls.ROM_MEM_DEVICE_ID_SHIFT
+        group_id = (header.flags & cls.ROM_MEM_GROUP_ID_MASK) >> cls.ROM_MEM_GROUP_ID_SHIFT
+        mem_id = get_memory_id(device_id, group_id)
+        return cls(header.address, header.count, header.flags, mem_id)
 
 
 class CmdReset(CmdBaseClass):
@@ -624,32 +681,43 @@ class CmdMemEnable(CmdBaseClass):
         self._header.count = value
 
     @property
-    def mem_type(self) -> MemId:
-        """Return memory to be enabled."""
-        return MemId.from_int(swap16(self._header.flags))
+    def flags(self) -> int:
+        """Return command's flag."""
+        return self._header.flags
 
-    @mem_type.setter
-    def mem_type(self, value: MemId) -> None:
-        """Setter.
+    @flags.setter
+    def flags(self, value: int) -> None:
+        """Set command's flag."""
+        self._header.flags = value
 
-        :param value: memory to be enabled
-        """
-        self._header.flags = swap16(value)
-
-    def __init__(self, address: int, size: int, mem_type: MemId):
+    def __init__(self, address: int, size: int, mem_id: int):
         """Initialize CmdMemEnable.
 
         :param address: source address with configuration data for memory initialization
         :param size: size of configuration data used for memory initialization
-        :param mem_type: identification of external memory type, see enum for details
+        :param mem_id: identification of memory
         """
         super().__init__(EnumCmdTag.MEM_ENABLE)
         self.address = address
-        self.mem_type = mem_type
+        self.mem_id = mem_id
         self.size = size
 
+        device_id = get_device_id(mem_id)
+        group_id = get_group_id(mem_id)
+
+        self.flags |= (self.flags & ~self.ROM_MEM_DEVICE_ID_MASK) | (
+            (device_id << self.ROM_MEM_DEVICE_ID_SHIFT) & self.ROM_MEM_DEVICE_ID_MASK
+        )
+
+        self.flags |= (self.flags & ~self.ROM_MEM_GROUP_ID_MASK) | (
+            (group_id << self.ROM_MEM_GROUP_ID_SHIFT) & self.ROM_MEM_GROUP_ID_MASK
+        )
+
     def __str__(self) -> str:
-        return f"MEM-ENABLE: Address=0x{self.address:08X}, Size={self.size}, MemType=0x{self.mem_type:08X}"
+        return (
+            f"MEM-ENABLE: Address=0x{self.address:08X}, Size={self.size}, "
+            f"Flags=0x{self.flags:08X}, MemId=0x{self.mem_id:08X}"
+        )
 
     @classmethod
     def parse(cls, data: bytes, offset: int = 0) -> "CmdMemEnable":
@@ -663,16 +731,104 @@ class CmdMemEnable(CmdBaseClass):
         header = CmdHeader.parse(data, offset)
         if header.tag != EnumCmdTag.MEM_ENABLE:
             raise SPSDKError("Invalid header tag")
-        return cls(header.address, header.count, MemId.from_int(swap16(header.flags)))
+        device_id = (header.flags & cls.ROM_MEM_DEVICE_ID_MASK) >> cls.ROM_MEM_DEVICE_ID_SHIFT
+        group_id = (header.flags & cls.ROM_MEM_GROUP_ID_MASK) >> cls.ROM_MEM_GROUP_ID_SHIFT
+        mem_id = get_memory_id(device_id, group_id)
+        return cls(header.address, header.count, mem_id)
 
 
 class CmdProg(CmdBaseClass):
     """Command Program class."""
 
-    def __init__(self) -> None:
-        """Initialize Cmd Program."""
+    @property
+    def address(self) -> int:
+        """Return address in target processor to program data."""
+        return self._header.address
+
+    @address.setter
+    def address(self, value: int) -> None:
+        """Setter.
+
+        :param value: address in target processor to load data
+        :raises SPSDKError: When there is incorrect address
+        """
+        if value < 0x00000000 or value > 0xFFFFFFFF:
+            raise SPSDKError("Incorrect address")
+        self._header.address = value
+
+    @property
+    def flags(self) -> int:
+        """Return command's flag."""
+        return self._header.flags
+
+    @flags.setter
+    def flags(self, value: int) -> None:
+        """Set command's flag."""
+        self._header.flags = self.is_eight_byte
+        self._header.flags |= value
+
+    @property
+    def data_word1(self) -> int:
+        """Return data word 1."""
+        return self._header.count
+
+    @data_word1.setter
+    def data_word1(self, value: int) -> None:
+        """Setter.
+
+        :param value: first data word
+        :raises SPSDKError: When there is incorrect value
+        """
+        if value < 0x00000000 or value > 0xFFFFFFFF:
+            raise SPSDKError("Incorrect data word 1")
+        self._header.count = value
+
+    @property
+    def data_word2(self) -> int:
+        """Return data word 2."""
+        return self._header.data
+
+    @data_word2.setter
+    def data_word2(self, value: int) -> None:
+        """Setter.
+
+        :param value: second data word
+        :raises SPSDKError: When there is incorrect value
+        """
+        if value < 0x00000000 or value > 0xFFFFFFFF:
+            raise SPSDKError("Incorrect data word 2")
+        self._header.data = value
+
+    def __init__(
+        self, address: int, mem_id: int, data_word1: int, data_word2: int = 0, flags: int = 0
+    ) -> None:
+        """Initialize CMD Prog."""
         super().__init__(EnumCmdTag.PROG)
 
+        if data_word2:
+            self.is_eight_byte = 1
+        else:
+            self.is_eight_byte = 0
+
+        if mem_id < 0 or mem_id > 0xFF:
+            raise SPSDKError("Invalid ID of memory")
+
+        self.address = address
+        self.data_word1 = data_word1
+        self.data_word2 = data_word2
+        self.mem_id = mem_id
+        self.flags = flags
+
+        self.flags = (self.flags & ~self.ROM_MEM_DEVICE_ID_MASK) | (
+            (self.mem_id << self.ROM_MEM_DEVICE_ID_SHIFT) & self.ROM_MEM_DEVICE_ID_MASK
+        )
+
+    def __str__(self) -> str:
+        return (
+            f"PROG: Index=0x{self.address:08X}, DataWord1=0x{self.data_word1:08X}, "
+            f"DataWord2=0x{self.data_word2:08X}, Flags=0x{self.flags:08X}, MemId=0x{self.mem_id:08X}"
+        )
+
     @classmethod
     def parse(cls, data: bytes, offset: int = 0) -> "CmdProg":
         """Parse command from bytes.
@@ -685,7 +841,8 @@ class CmdProg(CmdBaseClass):
         header = CmdHeader.parse(data, offset)
         if header.tag != EnumCmdTag.PROG:
             raise SPSDKError("Invalid header tag")
-        return cls()
+        mem_id = (header.flags & cls.ROM_MEM_DEVICE_ID_MASK) >> cls.ROM_MEM_DEVICE_ID_SHIFT
+        return cls(header.address, mem_id, header.count, header.data, header.flags)
 
 
 class VersionCheckType(Enum):
@@ -727,8 +884,8 @@ class CmdVersionCheck(CmdBaseClass):
 
     def __str__(self) -> str:
         return (
-            super().__str__()
-            + f" type={VersionCheckType.name(self.type)}, version={str(self.version)}"
+            f"CVER: Type={VersionCheckType.name(self.type)}, Version={str(self.version)}, "
+            f"Flags=0x{self.header.flags:08X}"
         )
 
     @classmethod
@@ -863,3 +1020,33 @@ def parse_command(data: bytes, offset: int = 0) -> CmdBaseClass:
     if header_tag not in _CMD_CLASS:
         raise SPSDKError(f"Unsupported command: {str(header_tag)}")
     return _CMD_CLASS[header_tag].parse(data, offset)
+
+
+def get_device_id(mem_id: int) -> int:
+    """Get device ID from memory ID.
+
+    :param mem_id: memory ID
+    :return: device ID
+    """
+    return ((mem_id) & DEVICE_ID_MASK) >> DEVICE_ID_SHIFT
+
+
+def get_group_id(mem_id: int) -> int:
+    """Get group ID from memory ID.
+
+    :param mem_id: memory ID
+    :return: group ID
+    """
+    return ((mem_id) & GROUP_ID_MASK) >> GROUP_ID_SHIFT
+
+
+def get_memory_id(device_id: int, group_id: int) -> int:
+    """Get memory ID from device ID and group ID.
+
+    :param device_id: device ID
+    :param group_id: group ID
+    :return: memory ID
+    """
+    return (((group_id) << GROUP_ID_SHIFT) & GROUP_ID_MASK) | (
+        ((device_id) << DEVICE_ID_SHIFT) & DEVICE_ID_MASK
+    )
