@@ -2,7 +2,7 @@
 # -*- coding: UTF-8 -*-
 #
 # Copyright 2017-2018 Martin Olejar
-# Copyright 2019-2021 NXP
+# Copyright 2019-2023 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 """Module for serial communication with a target device using SDP protocol."""
@@ -10,18 +10,20 @@
 import logging
 from typing import List, Optional, Union
 
-from serial import Serial
+from serial import Serial, SerialTimeoutException
 from serial.tools.list_ports import comports
 
 from spsdk.sdp.commands import CmdPacket, CmdResponse
 from spsdk.sdp.exceptions import SdpConnectionError
 
-from .base import Interface
+from .base import SDPInterface
 
 logger = logging.getLogger(__name__)
 
 
-def scan_uart(port: str = None, baudrate: int = None, timeout: int = None) -> List[Interface]:
+def scan_uart(
+    port: Optional[str] = None, baudrate: Optional[int] = None, timeout: Optional[int] = None
+) -> List["Uart"]:
     """Scan connected serial ports.
 
     Returns list of serial ports with devices that respond to PING command.
@@ -44,13 +46,13 @@ def scan_uart(port: str = None, baudrate: int = None, timeout: int = None) -> Li
     return list(filter(None, all_ports))
 
 
-def _check_port(port: str, baudrate: int, timeout: int) -> Optional[Interface]:
-    """Check if device on comport 'port' could be openned.
+def _check_port(port: str, baudrate: int, timeout: int) -> Optional["Uart"]:
+    """Check if device on comport 'port' could be opened.
 
     :param port: name of port to check
     :param baudrate: speed of the UART interface, defaults to 56700
     :param timeout: timeout in milliseconds
-    :return: None if device can't be openned, instance of Interface if it does
+    :return: None if device can't be opened, instance of Interface if it does
     """
     try:
         logger.debug(f"Checking port: {port}, baudrate: {baudrate}, timeout: {timeout}")
@@ -66,15 +68,15 @@ def _check_port(port: str, baudrate: int, timeout: int) -> Optional[Interface]:
 ########################################################################################################################
 
 
-class Uart(Interface):
+class Uart(SDPInterface):
     """UART interface."""
 
     @property
     def is_opened(self) -> bool:
-        """Return True if device is open, False othervise."""
+        """Return True if device is open, False otherwise."""
         return self.device.is_open
 
-    def __init__(self, port: str = None, timeout: int = 5000, baudrate: int = 115200):
+    def __init__(self, port: Optional[str] = None, timeout: int = 5000, baudrate: int = 115200):
         """Initialize the UART interface.
 
         :param port: name of the serial port, defaults to None
@@ -84,7 +86,10 @@ class Uart(Interface):
         """
         super().__init__()
         try:
-            self.device = Serial(port=port, timeout=timeout / 1000, baudrate=baudrate)
+            timeout_s = timeout / 1000
+            self.device = Serial(
+                port=port, timeout=timeout_s, write_timeout=timeout_s, baudrate=baudrate
+            )
             self.expect_status = True
         except Exception as e:
             raise SdpConnectionError(str(e)) from e
@@ -128,7 +133,7 @@ class Uart(Interface):
         :param config: parameters dictionary
         """
 
-    def read(self, length: int = None) -> CmdResponse:
+    def read(self, length: Optional[int] = None) -> CmdResponse:
         """Read data from device.
 
         :return: data read from device
@@ -170,6 +175,7 @@ class Uart(Interface):
         """Send data to device.
 
         :param data: Data to send
+        :raises TimeoutError: when sendig of data times-out
         :raises SdpConnectionError: when send data to device fails
         """
         logger.debug(f"[{' '.join(f'{b:02x}' for b in data)}]")
@@ -178,5 +184,9 @@ class Uart(Interface):
             self.device.reset_output_buffer()
             self.device.write(data)
             self.device.flush()
+        except SerialTimeoutException as e:
+            raise TimeoutError(
+                f"Write timeout error. The timeout is set to {self.device.write_timeout} s. Consider increasing it."
+            ) from e
         except Exception as e:
             raise SdpConnectionError(str(e)) from e
