@@ -1,13 +1,13 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2020-2021 NXP
+# Copyright 2020-2023 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 """The shadow registers control DAT support file."""
 import logging
-import math
-from typing import Any
+import os
+from typing import Any, List, Optional
 
 from ruamel.yaml import YAML
 from ruamel.yaml.comments import CommentedMap as CM
@@ -17,9 +17,9 @@ from spsdk.dat.debug_mailbox import DebugMailbox
 from spsdk.dat.dm_commands import StartDebugSession
 from spsdk.debuggers.debug_probe import DebugProbe, SPSDKDebugProbeError
 from spsdk.debuggers.utils import test_ahb_access
-from spsdk.utils.misc import change_endianism, reverse_bytes_in_longs, value_to_bytes
+from spsdk.utils.misc import change_endianness, value_to_bytes, value_to_int
 from spsdk.utils.reg_config import RegConfig
-from spsdk.utils.registers import Registers, RegsRegister
+from spsdk.utils.registers import Registers, RegsRegister, SPSDKRegsErrorRegisterNotFound
 
 logger = logging.getLogger(__name__)
 
@@ -33,7 +33,7 @@ class ShadowRegisters:
 
     def __init__(
         self,
-        debug_probe: DebugProbe,
+        debug_probe: Optional[DebugProbe],
         config: RegConfig,
         device: str,
         revision: str = "latest",
@@ -42,13 +42,18 @@ class ShadowRegisters:
         self.probe = debug_probe
         self.config = config
         self.device = device
-        self.offset = int(self.config.get_address(self.device, remove_underscore=True), 16)
+        self.offset = value_to_int(self.config.get_address(self.device))
+        self.fuse_mode = False
 
         self.regs = Registers(self.device)
         rev = revision or "latest"
-        rev = rev if rev != "latest" else config.get_latest_revision(self.device)
+        self.revision = (
+            rev
+            if rev != "latest"
+            else config.devices.get_by_name(self.device).revisions.get_latest().name
+        )
         self.regs.load_registers_from_xml(
-            config.get_data_file(self.device, rev),
+            config.get_data_file(self.device, self.revision),
             grouped_regs=config.get_grouped_registers(self.device),
         )
 
@@ -64,21 +69,27 @@ class ShadowRegisters:
             src.add_setvalue_hook(self.reg_antipolize_src_handler, dst)
             dst.add_setvalue_hook(self.reg_antipolize_dst_handler, src)
 
-    def _write_shadow_reg(self, addr: int, data: int, verify: int = True) -> None:
+    def _write_shadow_reg(self, addr: int, data: int, verify_mask: int = 0) -> None:
         """The function write a shadow register.
 
         The function writes shadow register in to MCU and verify the write if requested.
 
         param addr: Shadow register address.
         param data: Shadow register data to write.
-        param verify: If True the write is read back and compare, otherwise no check is done
+        param verify_mask: Verify bit mask for read back and compare, if 0 verify is disable
         raises IoVerificationError
         """
+        if not self.probe:
+            raise SPSDKDebugProbeError(
+                "Shadow registers: Cannot use the communication function without defined debug probe."
+            )
+
+        logger.debug(f"Writing shadow register address: {hex(addr)}, data: {hex(data)}")
         self.probe.mem_reg_write(addr, data)
 
-        if verify:
-            readback = self.probe.mem_reg_read(addr)
-            if readback != data:
+        if verify_mask:
+            read_back = self.probe.mem_reg_read(addr)
+            if read_back & verify_mask != data & verify_mask:
                 raise IoVerificationError(
                     f"The written data 0x{data:08X} to 0x{addr:08X} address are invalid."
                 )
@@ -91,14 +102,25 @@ class ShadowRegisters:
     def sets_all_registers(self) -> None:
         """Update all shadow registers in target by local values."""
         for reg in self.regs.get_registers():
-            self.set_register(reg.name, reg.get_value())
+            if reg.has_group_registers():
+                for subreg in reg.sub_regs:
+                    self.set_register(subreg.name, subreg.get_value())
+            else:
+                self.set_register(reg.name, reg.get_value())
 
     def reload_register(self, reg: RegsRegister) -> None:
         """Reload the value in requested register.
 
         :param reg: The register to reload from the HW.
         """
-        reg.set_value(self.get_register(reg.name))
+        reg.set_value(self.get_register(reg.name), raw=True)
+
+    def read_register(self, reg: RegsRegister) -> bytes:
+        """Read the value in requested register.
+
+        :param reg: The register to read from the HW.
+        """
+        return bytes(self.get_register(reg.name))
 
     def set_register(self, reg_name: str, data: Any) -> None:
         """The function sets the value of the specified register.
@@ -107,39 +129,45 @@ class ShadowRegisters:
         param data: The new data to be stored to shadow register.
         raises SPSDKDebugProbeError: The debug probe is not specified.
         """
-        if self.probe is None:
-            raise SPSDKDebugProbeError("There is no debug probe.")
 
-        try:
-            reg = self.regs.find_reg(reg_name)
-            value = value_to_bytes(data)
-
-            start_address = self.offset + reg.offset
-            width = reg.width
-
-            if width < len(value) * 8:
-                raise SPSDKError(f"Invalid length of data for shadow register write.")
-
-            if width < 32:
-                width = 32
-
-            data_aligned = bytearray(math.ceil(width / 8))
-            data_aligned[len(data_aligned) - len(value) : len(data_aligned)] = value
-
-            end_address = start_address + math.ceil(width / 8)
-            addresses = range(start_address, end_address, 4)
-
-            for i, addr in enumerate(addresses):
-                val = data_aligned[i * 4 : i * 4 + 4]
-                self._write_shadow_reg(
-                    addr,
-                    int.from_bytes(change_endianism(val) if reg.reverse else val, "big"),
+        def write_reg(base_address: int, reg: RegsRegister) -> None:
+            if not self.probe:
+                raise SPSDKDebugProbeError(
+                    "Shadow registers: Cannot use the communication function without defined debug probe."
                 )
 
-            reg.set_value(value, raw=True)
+            if reg.width > 32:
+                raise SPSDKError(
+                    f"Invalid width ({reg.width}b) of shadow register ({reg.name}) to write to device."
+                )
+            # Create value
+            value = reg.get_value()
+            if reg.reverse:
+                value = int.from_bytes(change_endianness(value_to_bytes(value)), "big")
+            # Create verify mask
+            bitfields = reg.get_bitfields()
+            verify_mask = 0
+            if bitfields:
+                for bitfield in bitfields:
+                    verify_mask = verify_mask | (((1 << bitfield.width) - 1) << bitfield.offset)
+            else:
+                verify_mask = (1 << reg.width) - 1
+
+            self._write_shadow_reg(
+                addr=base_address + reg.offset, data=value, verify_mask=verify_mask
+            )
+
+        try:
+            reg = self.regs.find_reg(reg_name, include_group_regs=True)
+            reg.set_value(data, raw=True)
+            if reg.has_group_registers():
+                for sub_reg in reg.sub_regs:
+                    write_reg(self.offset, sub_reg)
+            else:
+                write_reg(self.offset, reg=reg)
 
         except SPSDKError as exc:
-            raise SPSDKError(f"The get shadow register failed({str(exc)}).")
+            raise SPSDKError(f"The set shadow register failed({str(exc)}).") from exc
 
     def get_register(self, reg_name: str) -> bytes:
         """The function returns value of the requested register.
@@ -148,40 +176,88 @@ class ShadowRegisters:
         return: The value of requested register in bytes
         raises SPSDKDebugProbeError: The debug probe is not specified.
         """
-        if self.probe is None:
-            raise SPSDKDebugProbeError("There is no debug probe.")
 
-        array = bytearray()
+        def read_reg(base_address: int, reg: RegsRegister) -> bytes:
+            if not self.probe:
+                raise SPSDKDebugProbeError(
+                    "Shadow registers: Cannot use the communication function without defined debug probe."
+                )
+            if reg.width > 32:
+                raise SPSDKError(
+                    f"Invalid width ({reg.width}b) of shadow register ({reg.name}) to read from device."
+                )
+            read_value = self.probe.mem_reg_read(base_address + reg.offset).to_bytes(4, "big")
+            if reg.reverse:
+                read_value = change_endianness(read_value)
+            return read_value
+
         try:
-            reg = self.regs.find_reg(reg_name)
+            reg = self.regs.find_reg(reg_name, include_group_regs=True)
 
-            start_address = self.offset + reg.offset
-            width = reg.width
+            ret = bytearray()
+            if reg.has_group_registers():
 
-            if width < 32:
-                width = 32
-
-            if width == 32:
-                array.extend(self.probe.mem_reg_read(start_address).to_bytes(4, "big"))
+                for sub_reg in reg.sub_regs:
+                    ret.extend(read_reg(self.offset, sub_reg))
             else:
-                end_address = start_address + math.ceil(width / 8)
-                addresses = range(start_address, end_address, 4)
-
-                for addr in addresses:
-                    array.extend(self.probe.mem_reg_read(addr).to_bytes(4, "big"))
-
-            result = reverse_bytes_in_longs(bytes(array)) if reg.reverse else bytes(array)
+                ret.extend(read_reg(self.offset, reg=reg))
+            return ret
 
         except SPSDKError as exc:
-            raise SPSDKError(f"The get shadow register failed({str(exc)}).")
+            raise SPSDKError(f"The get shadow register failed({str(exc)}).") from exc
 
-        return result
+    def create_fuse_blhost_script(self, reg_list: List[str]) -> str:
+        """The function creates the BLHOST script to burn fuses.
 
-    def create_yml_config(self, file_name: str, raw: bool = False) -> None:
+        :param reg_list: The list of register to be burned.
+        :raises SPSDKError: Exception in case of not existing register.
+        :return: Content of BLHOST script file.
+        """
+
+        def add_reg(reg: RegsRegister) -> str:
+            otp_index = reg.otp_index
+            assert otp_index
+            otp_value = "0x" + reg.get_bytes_value().hex()
+            burn_fuse = f"# Fuse {reg.name}, index {otp_index} and value: {otp_value}.\n"
+            burn_fuse += f"efuse-program-once {hex(otp_index)} {otp_value}\n"
+            return burn_fuse
+
+        ret = (
+            "# BLHOST fuses programming script\n"
+            f"# Generated by SPSDK {__version__}\n"
+            f"# Chip: {self.device} rev:{self.revision}\n\n\n"
+        )
+        # Update list by antipole opposites registers
+        for ap_reg_src, ap_reg_dst in self.config.get_antipole_regs(self.device).items():
+            if ap_reg_src in reg_list:
+                reg_list.insert(reg_list.index(ap_reg_src) + 1, ap_reg_dst)
+        self.fuse_mode = True
+        for reg_name in reg_list:
+            try:
+                reg = self.regs.find_reg(reg_name, True)
+                # do recalculation based on fuse mode sets to ON!
+                reg.set_value(reg.get_value())
+            except SPSDKRegsErrorRegisterNotFound as exc:
+                self.fuse_mode = False
+                raise SPSDKError(
+                    f"Register {reg_name} has not found for {self.device} device."
+                ) from exc
+
+            if reg.has_group_registers():
+                for sub_reg in reg.sub_regs:
+                    ret += add_reg(sub_reg)
+            else:
+                ret += add_reg(reg)
+
+        self.fuse_mode = False
+        return ret
+
+    def create_yml_config(self, file_name: str, raw: bool = False, diff: bool = False) -> None:
         """The function creates the configuration YML file.
 
         :param file_name: The file_name (without extension) of stored configuration.
         :param raw: Raw output of configuration (including computed fields and anti-pole registers)
+        :param diff: Get only configuration with difference value to reset state.
         """
         antipole_regs = None if raw else list(self.config.get_antipole_regs(self.device).values())
         computed_fields = None if raw else self.config.get_computed_fields(self.device)
@@ -203,8 +279,15 @@ class ShadowRegisters:
 
         data["description"] = description
         data["registers"] = self.regs.create_yml_config(
-            exclude_regs=antipole_regs, exclude_fields=computed_fields, indent=2
+            exclude_regs=antipole_regs,
+            exclude_fields=computed_fields,
+            indent=2,
+            diff=diff,
         )
+
+        if not os.path.exists(os.path.dirname(file_name)):
+            os.makedirs(os.path.dirname(file_name))
+
         with open(file_name, "w", encoding="utf8") as out_file:
             yaml.dump(data, out_file)
 
@@ -222,8 +305,8 @@ class ShadowRegisters:
                 yaml = YAML()
                 yaml.indent(sequence=4, offset=2)
                 data = yaml.load(yml_config_file)
-        except FileNotFoundError:
-            raise SPSDKError("File with YML configuration doesn't exists.")
+        except FileNotFoundError as exc:
+            raise SPSDKError("File with YML configuration doesn't exists.") from exc
 
         self.regs.load_yml_config(data["registers"], antipole_regs, computed_fields)
         if not raw:
@@ -231,9 +314,10 @@ class ShadowRegisters:
             exclude_hooks = list(set(self.regs.get_reg_names()) - set(data["registers"].keys()))
             self.regs.run_hooks(exclude_hooks)
 
-        logger.debug(f"The shadow registers has been loaded from configuration.")
+        logger.debug("The shadow registers has been loaded from configuration.")
 
-    def reg_antipolize_src_handler(self, val: bytes, context: Any) -> bytes:
+    @staticmethod
+    def reg_antipolize_src_handler(val: int, context: Any) -> int:
         """Antipolize given register value.
 
         :param val: Input register value.
@@ -241,14 +325,11 @@ class ShadowRegisters:
         :return: Antipolized value.
         """
         dst_reg: RegsRegister = context
-        newval = [0] * len(val)
-        for i, val_byte in enumerate(val):
-            newval[i] = val_byte ^ 0xFF
-        dst_reg.set_value(bytes(newval), raw=True)
-
+        dst_reg.set_value(val ^ 0xFFFFFFFF, raw=True)
         return val
 
-    def reg_antipolize_dst_handler(self, val: bytes, context: Any) -> bytes:
+    @staticmethod
+    def reg_antipolize_dst_handler(val: int, context: Any) -> int:
         """Keep same antipolized register value in computed register.
 
         :param val: Input register value.
@@ -257,10 +338,8 @@ class ShadowRegisters:
         """
         src_reg: RegsRegister = context
         val = src_reg.get_value()
-        newval = [0] * len(val)
-        for i, val_byte in enumerate(val):
-            newval[i] = val_byte ^ 0xFF
-        return bytes(newval)
+        new_val = val ^ 0xFFFFFFFF
+        return new_val
 
     def reg_computed_fields_handler(self, val: bytes, context: Any) -> bytes:
         """Recalculate all fields for given register value.
@@ -273,7 +352,7 @@ class ShadowRegisters:
         fields: dict = context
         for method in fields.values():
             if hasattr(self, method):
-                method_ref = getattr(self, method, None)
+                method_ref = getattr(self, method)
                 val = method_ref(val)
             else:
                 raise SPSDKError(f"The '{method}' compute function doesn't exists.")
@@ -309,33 +388,32 @@ class ShadowRegisters:
         return crc & 0xFF
 
     @staticmethod
-    def comalg_dcfg_cc_socu_crc8(val: bytes) -> bytes:
+    def comalg_dcfg_cc_socu_crc8(val: int) -> int:
         """Function that creates the crc for DCFG_CC_SOCU.
 
         :param val: Input DCFG_CC_SOCU Value.
         :return: Returns the value of DCFG_CC_SOCU with computed CRC8 field.
         """
-        ret = [0] * 4
-        ret[0:3] = val[0:3]
-        in_val = bytearray(val[0:3])
-        in_val.reverse()
-        ret[3] = ShadowRegisters.crc_update(in_val)
-        return bytes(ret)
+        in_val = bytearray(3)
+        for i in range(3):
+            in_val[i] = (val >> (8 + i * 8)) & 0xFF
+        val &= ~0xFF
+        val |= ShadowRegisters.crc_update(in_val)
+        return val
 
-    @staticmethod
-    def comalg_dcfg_cc_socu_rsvd(val: bytes) -> bytes:
-        """Function fill up the DCFG_CC_SOCU RSVD filed by 0x80 to satisfy MCU needs.
+    def comalg_dcfg_cc_socu_test_en(self, val: int) -> int:
+        """Function fill up the DCFG_CC_SOCU DEV_TEST_EN set to True to satisfy MCU needs.
 
         :param val: Input DCFG_CC_SOCU Value.
-        :return: Returns the value of DCFG_CC_SOCU with computed CRC8 field.
+        :return: Returns the value of DCFG_CC_SOCU with optionally enabled test mode.
         """
-        new_val = bytearray(val)
-        new_val[0] &= ~0xFE
-        new_val[0] |= 0x80
-        return new_val
+        if self.fuse_mode:
+            return val & ~0x80000000
+
+        return val | 0x80000000
 
     @staticmethod
-    def comalg_do_nothing(val: bytes) -> bytes:
+    def comalg_do_nothing(val: int) -> int:
         """Function that do nothing.
 
         :param val: Input Value.
