@@ -1,14 +1,15 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2019-2021 NXP
+# Copyright 2019-2023 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
 """Internal implementation for security backend."""
 import importlib
+import math
 from struct import pack, unpack_from
-from typing import Any, Union
+from typing import Any, Optional, Union
 
 # Used security modules
 from Crypto import Hash, Random
@@ -18,6 +19,7 @@ from Crypto.PublicKey import ECC, RSA
 from Crypto.Signature import DSS, pkcs1_15
 
 from spsdk import SPSDKError
+from spsdk.crypto import PrivateKey
 
 # Abstract Class Interface
 from .abstract import BackendClass
@@ -59,8 +61,8 @@ class Backend(BackendClass):
         # algo_cls = getattr(Hash, name.upper(), None)  # hack: get class object by name
         try:
             algo_cls = importlib.import_module(f"Crypto.Hash.{name.upper()}")
-        except ModuleNotFoundError as e:
-            raise SPSDKError(f"No module named 'Crypto.Hash.{name.upper()}")
+        except ModuleNotFoundError as exc:
+            raise SPSDKError(f"No module named 'Crypto.Hash.{name.upper()}") from exc
         if algo_cls is None:
             raise SPSDKError(f"Unsupported algorithm: Hash.{name}".format(name=name.upper()))
         return algo_cls.new(data)  # type: ignore  # pylint: disable=not-callable
@@ -159,12 +161,14 @@ class Backend(BackendClass):
             raise SPSDKError(f"Integrity Check Failed: {a:016X} (expected {iv:016X})")
         return b"".join(r[1:])
 
-    def aes_cbc_encrypt(self, key: bytes, plain_data: bytes, iv: bytes = None) -> bytes:
+    def aes_cbc_encrypt(
+        self, key: bytes, plain_data: bytes, iv_data: Optional[bytes] = None
+    ) -> bytes:
         """Encrypt plain data with AES in CBC mode.
 
         :param key: Key for encryption
         :param plain_data: Data to encrypt
-        :param iv: Initial vector for encryption, defaults to None
+        :param iv_data: Initial vector for encryption, defaults to None
         :return: Encrypted data
         :raises SPSDKError: Incorrect key or initialization vector size
         """
@@ -172,12 +176,33 @@ class Backend(BackendClass):
             raise SPSDKError(
                 f"The key must be a valid AES key length: {', '.join([str(k) for k in AES.key_size])}"
             )
-        init_vector = iv or bytes(AES.block_size)
+        init_vector = iv_data or bytes(AES.block_size)
         if len(init_vector) != AES.block_size:
             raise SPSDKError(f"The initial vector length must be {AES.block_size}")
         cipher = AES.new(key, mode=AES.MODE_CBC, iv=init_vector)
         return cipher.encrypt(plain_data)
 
+    def aes_cbc_decrypt(
+        self, key: bytes, encrypted_data: bytes, iv_data: Optional[bytes] = None
+    ) -> bytes:
+        """Decrypt encrypted data with AES in CBC mode.
+
+        :param key: The key for data decryption
+        :param encrypted_data: Input data
+        :param iv_data: Initialization vector data
+        :raises SPSDKError: Invalid Key or IV
+        :return: Decrypted image
+        """
+        if len(key) not in AES.key_size:
+            raise SPSDKError(
+                f"The key must be a valid AES key length: {', '.join([str(k) for k in AES.key_size])}"
+            )
+        init_vector = iv_data or bytes(AES.block_size)
+        if len(init_vector) != AES.block_size:
+            raise SPSDKError(f"The initial vector length must be {AES.block_size}")
+        cipher = AES.new(key, mode=AES.MODE_CBC, iv=init_vector)
+        return cipher.decrypt(encrypted_data)
+
     def aes_ctr_encrypt(self, key: bytes, plain_data: bytes, nonce: bytes) -> bytes:
         """Encrypt plain data with AES in CTR mode.
 
@@ -209,6 +234,28 @@ class Backend(BackendClass):
         """
         return self.aes_ctr_encrypt(key, encrypted_data, nonce)
 
+    def aes_xts_encrypt(self, key: bytes, plain_data: bytes, tweak: bytes) -> bytes:
+        """Encrypt plain data with AES in XTS mode.
+
+        :param key: The key for data encryption
+        :param plain_data: Input data
+        :param tweak: The tweak is a 16 byte value
+        :return: Encrypted data
+        :raises NotImplementedError: This backend doesn't support AES-XTS
+        """
+        raise NotImplementedError("This backend doesn't support AES-XTS")
+
+    def aes_xts_decrypt(self, key: bytes, encrypted_data: bytes, tweak: bytes) -> bytes:
+        """Decrypt encrypted data with AES in XTS mode.
+
+        :param key: The key for data decryption
+        :param encrypted_data: Input data
+        :param tweak: The tweak is a 16 byte value
+        :return: Decrypted data
+        :raises NotImplementedError: This backend doesn't support AES-XTS
+        """
+        raise NotImplementedError("This backend doesn't support AES-XTS")
+
     def rsa_sign(
         self,
         private_key: Union[RSA.RsaKey, bytes],
@@ -267,7 +314,7 @@ class Backend(BackendClass):
         return RSA.construct((modulus, exponent))
 
     def ecc_sign(
-        self, private_key: Union[ECC.EccKey, bytes], data: bytes, algorithm: str = None
+        self, private_key: Union[ECC.EccKey, bytes], data: bytes, algorithm: Optional[str] = None
     ) -> bytes:
         """Sign data using (EC)DSA.
 
@@ -284,10 +331,10 @@ class Backend(BackendClass):
 
     def ecc_verify(
         self,
-        key: Union[ECC.EccKey, bytes],
+        key: Union[ECC.EccKey, bytes],  # TODO  - could we renamed abstract class to "key" only?
         signature: bytes,
         data: bytes,
-        algorithm: str = None,
+        algorithm: Optional[str] = None,
     ) -> bool:
         """Verify (EC)DSA signature.
 
@@ -312,6 +359,22 @@ class Backend(BackendClass):
         except ValueError:
             return False
 
+    @staticmethod
+    def sign_size(key: PrivateKey) -> int:
+        """Get size of signature for loaded private key.
+
+        :param key: Private key used to sign data.
+        :return: Size of signature in bytes for the private key.
+        :raises SPSDKError: Invalid key type.
+        """
+        if isinstance(key, ECC.EccKey):
+            return math.ceil(key.key_size / 8) * 2
+
+        if isinstance(key, RSA.RsaKey):
+            return key.key_size // 8
+
+        raise SPSDKError(f"Unsupported private key type to get signature size. {type(key)}")
+
 
 ########################################################################################################################
 # SPSDK Backend instance
