@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: UTF-8 -*-
 #
-# Copyright 2019-2021 NXP
+# Copyright 2019-2023 NXP
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
@@ -11,7 +11,7 @@ import math
 
 # Used security modules
 from secrets import token_bytes
-from typing import Any, Union
+from typing import Any, Optional, Union
 
 from cryptography.exceptions import InvalidSignature
 from cryptography.hazmat.backends import default_backend
@@ -20,6 +20,8 @@ from cryptography.hazmat.primitives.asymmetric import ec, padding, rsa, utils
 from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes
 
 from spsdk import SPSDKError
+from spsdk.crypto import EllipticCurvePrivateKey, PrivateKey, RSAPrivateKey
+from spsdk.utils.misc import align_block
 
 # Abstract Class Interface
 from .abstract import BackendClass
@@ -106,6 +108,55 @@ class Backend(BackendClass):
         """
         return keywrap.aes_key_unwrap(kek, wrapped_key, default_backend())
 
+    def aes_cbc_encrypt(
+        self, key: bytes, plain_data: bytes, iv_data: Optional[bytes] = None
+    ) -> bytes:
+        """Encrypt plain data with AES in CBC mode.
+
+        :param key: The key for data encryption
+        :param plain_data: Input data
+        :param iv_data: Initialization vector data
+        :raises SPSDKError: Invalid Key or IV
+        :return: Encrypted image
+        """
+        if len(key) * 8 not in algorithms.AES.key_sizes:
+            raise SPSDKError(
+                "The key must be a valid AES key length: "
+                f"{', '.join([str(k) for k in algorithms.AES.key_sizes])}"
+            )
+        init_vector = iv_data or bytes(algorithms.AES.block_size)
+        if len(init_vector) * 8 != algorithms.AES.block_size:
+            raise SPSDKError(f"The initial vector length must be {algorithms.AES.block_size}")
+        cipher = Cipher(algorithms.AES(key), modes.CBC(init_vector), default_backend())
+        enc = cipher.encryptor()
+        return (
+            enc.update(align_block(plain_data, alignment=algorithms.AES.block_size // 8))
+            + enc.finalize()
+        )
+
+    def aes_cbc_decrypt(
+        self, key: bytes, encrypted_data: bytes, iv_data: Optional[bytes] = None
+    ) -> bytes:
+        """Decrypt encrypted data with AES in CBC mode.
+
+        :param key: The key for data decryption
+        :param encrypted_data: Input data
+        :param iv_data: Initialization vector data
+        :raises SPSDKError: Invalid Key or IV
+        :return: Decrypted image
+        """
+        if len(key) * 8 not in algorithms.AES.key_sizes:
+            raise SPSDKError(
+                "The key must be a valid AES key length: "
+                f"{', '.join([str(k) for k in algorithms.AES.key_sizes])}"
+            )
+        init_vector = iv_data or bytes(algorithms.AES.block_size)
+        if len(init_vector) * 8 != algorithms.AES.block_size:
+            raise SPSDKError(f"The initial vector length must be {algorithms.AES.block_size}")
+        cipher = Cipher(algorithms.AES(key), modes.CBC(init_vector), default_backend())
+        dec = cipher.decryptor()
+        return dec.update(encrypted_data) + dec.finalize()
+
     def aes_ctr_encrypt(self, key: bytes, plain_data: bytes, nonce: bytes) -> bytes:
         """Encrypt plain data with AES in CTR mode.
 
@@ -130,6 +181,30 @@ class Backend(BackendClass):
         enc = cipher.decryptor()
         return enc.update(encrypted_data) + enc.finalize()
 
+    def aes_xts_encrypt(self, key: bytes, plain_data: bytes, tweak: bytes) -> bytes:
+        """Encrypt plain data with AES in XTS mode.
+
+        :param key: The key for data encryption
+        :param plain_data: Input data
+        :param tweak: The tweak is a 16 byte value
+        :return: Encrypted data
+        """
+        cipher = Cipher(algorithms.AES(key), modes.XTS(tweak), default_backend())
+        enc = cipher.encryptor()
+        return enc.update(plain_data) + enc.finalize()
+
+    def aes_xts_decrypt(self, key: bytes, encrypted_data: bytes, tweak: bytes) -> bytes:
+        """Decrypt encrypted data with AES in XTS mode.
+
+        :param key: The key for data decryption
+        :param encrypted_data: Input data
+        :param tweak: The tweak is a 16 byte value
+        :return: Decrypted data
+        """
+        cipher = Cipher(algorithms.AES(key), modes.XTS(tweak), default_backend())
+        enc = cipher.decryptor()
+        return enc.update(encrypted_data) + enc.finalize()
+
     def rsa_sign(
         self,
         private_key: Union[rsa.RSAPrivateKey, bytes],
@@ -145,9 +220,13 @@ class Backend(BackendClass):
         :raises SPSDKError: If algorithm not found
         """
         if isinstance(private_key, bytes):
-            private_key = serialization.load_pem_private_key(private_key, None, default_backend())
-        assert isinstance(private_key, rsa.RSAPrivateKey)
-        return private_key.sign(
+            processed_private_key = serialization.load_pem_private_key(
+                private_key, None, default_backend()
+            )
+        if isinstance(private_key, rsa.RSAPrivateKey):
+            processed_private_key = private_key
+        assert isinstance(processed_private_key, rsa.RSAPrivateKey)
+        return processed_private_key.sign(
             data=data,
             padding=padding.PKCS1v15(),
             algorithm=self._get_algorithm(algorithm),
@@ -198,7 +277,7 @@ class Backend(BackendClass):
         self,
         private_key: Union[ec.EllipticCurvePrivateKey, bytes],
         data: bytes,
-        algorithm: str = None,
+        algorithm: Optional[str] = None,
     ) -> bytes:
         """Sign data using (EC)DSA.
 
@@ -208,15 +287,21 @@ class Backend(BackendClass):
         :return: Signature, r and s coordinates as bytes
         """
         if isinstance(private_key, bytes):
-            private_key = serialization.load_pem_private_key(private_key, None, default_backend())
-        assert isinstance(private_key, ec.EllipticCurvePrivateKey)
-        hash_name = algorithm or f"sha{private_key.key_size}"
-        der_signature = private_key.sign(
-            data, signature_algorithm=ec.ECDSA(self._get_algorithm(hash_name))
+            processed_private_key = serialization.load_pem_private_key(
+                private_key, None, default_backend()
+            )
+        if isinstance(private_key, ec.EllipticCurvePrivateKey):
+            processed_private_key = private_key
+        assert isinstance(processed_private_key, ec.EllipticCurvePrivateKey)
+        hash_name = (
+            algorithm
+            or {256: "sha256", 384: "sha384", 521: "sha512"}[processed_private_key.key_size]
         )
+        # pylint: disable=no-value-for-parameter    # pylint is mixing RSA and ECC sing methods
+        der_signature = processed_private_key.sign(data, ec.ECDSA(self._get_algorithm(hash_name)))
         # pylint: disable=invalid-name  # we want to use established names
         r, s = utils.decode_dss_signature(der_signature)
-        coordinate_size = math.ceil(private_key.key_size / 8)
+        coordinate_size = math.ceil(processed_private_key.key_size / 8)
         r_bytes = r.to_bytes(coordinate_size, byteorder="big")
         s_bytes = s.to_bytes(coordinate_size, byteorder="big")
         return r_bytes + s_bytes
@@ -226,7 +311,7 @@ class Backend(BackendClass):
         public_key: Union[ec.EllipticCurvePublicKey, bytes],
         signature: bytes,
         data: bytes,
-        algorithm: str = None,
+        algorithm: Optional[str] = None,
     ) -> bool:
         """Verify (EC)DSA signature.
 
@@ -238,24 +323,45 @@ class Backend(BackendClass):
         :raises SPSDKError: Signature length is invalid
         """
         if isinstance(public_key, bytes):
-            public_key = serialization.load_pem_public_key(public_key, default_backend())
-        assert isinstance(public_key, ec.EllipticCurvePublicKey)
-        coordinate_size = math.ceil(public_key.key_size / 8)
+            processed_public_key = serialization.load_pem_public_key(public_key, default_backend())
+        if isinstance(public_key, ec.EllipticCurvePublicKey):
+            processed_public_key = public_key
+        assert isinstance(processed_public_key, ec.EllipticCurvePublicKey)
+        coordinate_size = math.ceil(processed_public_key.key_size / 8)
         if len(signature) != 2 * coordinate_size:
             raise SPSDKError(
                 f"Invalid signature size: expected {2 * coordinate_size}, actual: {len(signature)}"
             )
-        hash_name = algorithm or f"sha{public_key.key_size}"
+        hash_name = algorithm or f"sha{processed_public_key.key_size}"
         der_signature = utils.encode_dss_signature(
             int.from_bytes(signature[:coordinate_size], byteorder="big"),
             int.from_bytes(signature[coordinate_size:], byteorder="big"),
         )
         try:
-            public_key.verify(der_signature, data, ec.ECDSA(self._get_algorithm(hash_name)))
+            # pylint: disable=no-value-for-parameter    # pylint is mixing RSA and ECC verify methods
+            processed_public_key.verify(
+                der_signature, data, ec.ECDSA(self._get_algorithm(hash_name))
+            )
             return True
         except InvalidSignature:
             return False
 
+    @staticmethod
+    def sign_size(key: PrivateKey) -> int:
+        """Get size of signature for loaded private key.
+
+        :param key: Private key used to sign data.
+        :return: Size of signature in bytes for the private key.
+        :raises SPSDKError: Invalid key type.
+        """
+        if isinstance(key, EllipticCurvePrivateKey):
+            return math.ceil(key.key_size / 8) * 2
+
+        if isinstance(key, RSAPrivateKey):
+            return key.key_size // 8
+
+        raise SPSDKError(f"Unsupported private key type to get signature size. {type(key)}")
+
 
 ########################################################################################################################
 # SPSDK OpenSSL Backend instance
